import { __awaiter, __decorate, __param, __metadata } from 'tslib';
import { NgZone, Optional, Injectable, Input, Component, ViewEncapsulation, ChangeDetectorRef, forwardRef, EventEmitter, Output, ElementRef, Renderer2, HostListener, Directive, ChangeDetectionStrategy, NgModule, TemplateRef, ContentChildren, QueryList, ViewChild, ViewContainerRef } from '@angular/core';
import Swagger from 'swagger-client/browser/index';
import { HttpClient, HttpHeaders, HttpRequest, HttpEventType, HttpErrorResponse, HttpClientModule } from '@angular/common/http';
import { BehaviorSubject, Subject, throwError, Observable, ReplaySubject, combineLatest } from 'rxjs';
import { map, takeUntil, filter, debounceTime } from 'rxjs/operators';
import { CommonModule } from '@angular/common';
import { FormControl, FormGroup, Validators, FormArray, NG_VALUE_ACCESSOR, FormsModule, ReactiveFormsModule } from '@angular/forms';
import { NavigationEnd, Router, ActivatedRoute, RouterModule } from '@angular/router';
import { MarkdownModule } from 'ngx-markdown';
import { CdkPortal, PortalModule } from '@angular/cdk/portal';
import { Overlay, OverlayModule } from '@angular/cdk/overlay';
import { AceModule } from 'ngx-ace-wrapper';
import 'brace';
import 'brace/mode/json';
import 'brace/mode/xml';
import * as Ajv from 'ajv';

const httpMethods = ['get', 'put', 'post', 'delete', 'options', 'head', 'trace'];
const securitySchemeTypes = ['apiKey', 'http', 'oauth2', 'openIdConnect'];
const securitySchemeTypesWithScopes = ['oauth2', 'openIdConnect'];

const formatExampleDate = new Date();
formatExampleDate.setMilliseconds(0);
formatExampleDate.setSeconds(0);
const ɵ0 = () => '', ɵ1 = () => 'user@example.com', ɵ2 = () => formatExampleDate.toISOString(), ɵ3 = () => formatExampleDate.toISOString().substring(0, 10), ɵ4 = () => formatExampleDate.toISOString().substring(11), ɵ5 = () => '3fa85f64-5717-4562-b3fc-2c963f66afa6', ɵ6 = () => 'example.com', ɵ7 = () => '198.51.100.42', ɵ8 = () => '2001:0db8:5b96:0000:0000:426f:8e17:642a', ɵ9 = () => 0, ɵ10 = () => 0.0, ɵ11 = () => 0, ɵ12 = schema => (typeof schema.default === 'boolean' ? schema.default : true), ɵ13 = () => null, ɵ14 = () => null;
const formatExamples = {
    string: ɵ0,
    string_email: ɵ1,
    'string_date-time': ɵ2,
    string_date: ɵ3,
    string_time: ɵ4,
    string_uuid: ɵ5,
    string_hostname: ɵ6,
    string_ipv4: ɵ7,
    string_ipv6: ɵ8,
    number: ɵ9,
    number_float: ɵ10,
    integer: ɵ11,
    boolean: ɵ12,
    null: ɵ13,
    any: ɵ14
};
function examplePrimitive(schema) {
    const type = Array.isArray(schema.type) ? schema.type[0] : schema.type;
    const formatFunc = formatExamples[`${type}_${schema.format}`] || formatExamples[type];
    if (typeof formatFunc === 'function') {
        return formatFunc(schema);
    }
    return `Unknown: ` + type;
}
function exampleFromSchema(schema) {
    const type = Array.isArray(schema.type) ? schema.type[0] : schema.type;
    if ('const' in schema) {
        return schema.const;
    }
    if ('default' in schema) {
        return schema.default;
    }
    if ('examples' in schema && schema.examples.length) {
        return schema.examples[0];
    }
    if (schema.enum) {
        return schema.enum[0];
    }
    if (type === 'object' || schema.properties) {
        const result = {};
        if (schema.example) {
            return schema.example;
        }
        if (!schema.properties) {
            return result;
        }
        const keys = Object.keys(schema.properties);
        for (const key of keys) {
            result[key] = exampleFromSchema(schema.properties[key]);
        }
        return result;
    }
    if (type === 'array') {
        const item = Array.isArray(schema.items) ? schema.items[0] : schema.items;
        return [exampleFromSchema(item)];
    }
    return examplePrimitive(schema);
}
function randomHex(len) {
    const preHex = len - 1 === 0 ? 0 : Math.pow(2, (4 * (len - 1)));
    const hex = Math.pow(2, (4 * len)) - 1;
    return Math.floor(preHex + Math.random() * (hex - preHex)).toString(16);
}
function getStarMatcher(pattern) {
    const escaped = escapeRegExpNoStar(pattern);
    return new RegExp('^' + escaped.replace(/\*/g, '.{0,50}') + '$');
}
function escapeRegExpNoStar(pattern) {
    return pattern.replace(/[.+?^${}()|[\]\\]/g, '\\$&');
}
function getJsonString(input) {
    if (typeof input === 'string' && input.match(/^({|\[|"|\d|-\d|null)/)) {
        return input;
    }
    return JSON.stringify(input, null, 2);
}
function serializeQueryParams(queryParams) {
    if (!queryParams) {
        return '';
    }
    return Object.keys(queryParams)
        .map(key => encodeURIComponent(key) + '=' + encodeURIComponent(queryParams[key]))
        .join('&');
}
function parseQueryString(queryParams) {
    if (typeof queryParams !== 'string' || !queryParams.length) {
        return {};
    }
    const paramsList = queryParams.split('&');
    const params = {};
    if (paramsList.length) {
        for (const line of paramsList) {
            const [key, value] = line.split('=');
            params[decodeURIComponent(key)] = decodeURIComponent(value);
        }
    }
    return params;
}

let OpenapiAuthService = class OpenapiAuthService {
    constructor(zone, http) {
        this.zone = zone;
        this.http = http;
        this.securitySchemes = new BehaviorSubject([]);
        /**
         * Emits with the scheme name when its credentials have been updated
         */
        this.updatedCredentials = new Subject();
        this.globalRequirements = [];
        this.currentWindow = window;
    }
    identifySchemes(spec) {
        this.securitySchemes.next(getSecurityInformation(spec));
        if (Array.isArray(spec.security)) {
            this.globalRequirements = spec.security;
        }
    }
    getAllRequirements(requirement) {
        return [...this.globalRequirements, ...(requirement || [])].filter(reqObject => Object.keys(reqObject).length > 0);
    }
    getRequiredSchemes(requirement) {
        const result = [];
        for (const reqs of this.getAllRequirements(requirement)) {
            result.push(Object.keys(reqs).map(reqName => {
                const schemaItem = this.getSchema(reqName);
                return {
                    name: reqName,
                    scopes: reqs[reqName],
                    securityScheme: schemaItem.securityScheme,
                    authenticated: schemaItem.authenticated
                };
            }));
        }
        return result;
    }
    getAuthStatus(requirements) {
        let authStatus = 'none';
        const allRequirements = this.getAllRequirements(requirements);
        if (allRequirements && Object.keys(allRequirements).length) {
            authStatus = 'required';
            if (this.hasAnyAuthorization(allRequirements)) {
                authStatus = 'ok';
            }
        }
        return authStatus;
    }
    hasAnyAuthorization(requirements) {
        if (!requirements || !requirements.length) {
            return true;
        }
        // one needs to be fulfilled
        for (const req of requirements) {
            const reqNames = Object.keys(req);
            if (reqNames.length && reqNames.every(rn => this.hasRequirement(rn, req[rn]))) {
                return true;
            }
        }
        return false;
    }
    hasRequirement(reqName, scopes) {
        const scheme = this.getSchema(reqName);
        if (scheme &&
            scheme.authenticated &&
            scheme.securityScheme.type &&
            securitySchemeTypesWithScopes.includes(scheme.securityScheme.type)) {
            const credentials = scheme.credentials;
            if (credentials && credentials.scopes && scopes.every(s => credentials.scopes.includes(s))) {
                return true;
            }
        }
        else if (scheme && scheme.authenticated) {
            return true;
        }
        return false;
    }
    getSchema(name) {
        return this.securitySchemes.value.find(s => s.name === name);
    }
    updateCredentials(name, credentials, authenticated, remember = false) {
        if (typeof name !== 'string' || !name.length) {
            throw new Error('No credentials name given');
        }
        const schema = this.getSchema(name);
        schema.authenticated = authenticated;
        if (schema.authenticated) {
            schema.credentials = credentials;
            if (remember) {
                schema.remember = true;
                writeCredentialsStore(name, credentials);
            }
        }
        else {
            schema.remember = false;
            schema.credentials = null;
            clearCredentialsStore(name);
        }
        this.updatedCredentials.next(name);
    }
    runOAuthAuthorization(name, credentials) {
        const scheme = this.getSchema(name);
        if (!scheme || !credentials) {
            return throwError(new Error('Unknown scheme or no credentials given'));
        }
        scheme.credentials = credentials;
        scheme.remember = true;
        writeCredentialsStore(name, credentials);
        if (credentials.flow === 'implicit' || credentials.flow === 'authorizationCode') {
            return new Observable(subscriber => {
                const url = buildAuthorizationUrl(scheme.securityScheme, credentials, name);
                const service = this;
                this.currentWindow.handleOAuthCallback = fragment => {
                    // called from other window
                    return service.zone.run(() => service.handleOAuthCallback(fragment).then(() => {
                        subscriber.next(true);
                        subscriber.complete();
                    }, err => {
                        subscriber.error(err);
                    }));
                };
                this.currentWindow.open(url, '_blank');
            });
        }
        if (credentials.flow === 'clientCredentials') {
            const tokenUrl = getTokenBaseUrl(scheme.securityScheme, 'clientCredentials');
            return this.requestToken(tokenUrl, 'client_credentials', credentials).pipe(map(token => {
                credentials.token = token;
                this.updateCredentials(name, credentials, true, true);
                return true;
            }));
        }
        if (credentials.flow === 'password') {
            const tokenUrl = getTokenBaseUrl(scheme.securityScheme, 'password');
            return this.requestToken(tokenUrl, 'password', credentials).pipe(map(token => {
                credentials.token = token;
                this.updateCredentials(name, credentials, true, true);
                return true;
            }));
        }
        return throwError(new Error('Unknown flow'));
    }
    /**
     * @param params
     * eg. access_token=8c665834-6a43-4704-81d3-42d311a1ab91&token_type=bearer&expires_in=20&scope=read%20write:pets%20write%20read:pets
     */
    handleOAuthCallback(params) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.currentWindow.opener && this.currentWindow.opener.handleOAuthCallback) {
                yield this.currentWindow.opener.handleOAuthCallback(params);
                this.currentWindow.close();
                return;
            }
            if (!params.state) {
                return;
            }
            const scheme = this.getSchema(params.state);
            if (!scheme || !scheme.credentials) {
                return;
            }
            const credentials = scheme.credentials;
            if (params.access_token) {
                return this.updateByImplicitFlow(scheme.name, credentials, params);
            }
            if (params.code) {
                return yield this.updateByCodeFlow(scheme, credentials, params);
            }
        });
    }
    updateByImplicitFlow(name, credentials, params) {
        credentials.token = {
            access_token: params.access_token,
            token_type: params.token_type
        };
        if (params.expires_in) {
            credentials.token.expires_at = new Date().getTime() + parseInt(params.expires_in, 10) * 1000;
        }
        this.updateCredentials(name, credentials, true, true);
    }
    updateByCodeFlow(scheme, credentials, params) {
        return __awaiter(this, void 0, void 0, function* () {
            const tokenUrl = getTokenBaseUrl(scheme.securityScheme, 'authorizationCode');
            credentials.token = yield this.requestToken(tokenUrl, 'authorization_code', credentials, params.code).toPromise();
            this.updateCredentials(name, credentials, true, true);
        });
    }
    /**
     * Updates the provided credentials with a token
     */
    requestToken(tokenUrl, grantType, credentials, code) {
        const tokenRequest = {
            grant_type: grantType,
            client_id: credentials.clientId,
            client_secret: credentials.clientSecret
        };
        if (grantType === 'authorization_code') {
            tokenRequest.code = code;
            tokenRequest.redirect_uri = credentials.redirectUri;
        }
        if (grantType === 'password') {
            tokenRequest.username = credentials.username;
            tokenRequest.password = credentials.password;
        }
        if (!this.http) {
            throw new Error('No http client provider available.');
        }
        return this.http
            .post(tokenUrl, serializeQueryParams(tokenRequest), {
            headers: {
                'Content-type': 'application/x-www-form-urlencoded'
            }
        })
            .pipe(map(result => {
            if (result && result.access_token) {
                const token = {
                    access_token: result.access_token,
                    token_type: result.token_type,
                    id_token: result.id_token,
                    refresh_token: result.refresh_token
                };
                if (result.expires_in) {
                    token.expires_at = new Date().getTime() + parseInt(result.expires_in, 10) * 1000;
                }
                return token;
            }
            else {
                throw new Error('Missing access token in response from token endpoint ' + tokenUrl);
            }
        }));
    }
    removeToken(name) {
        const schema = this.getSchema(name);
        if (!schema || !schema.credentials) {
            return;
        }
        schema.credentials.token = null;
        schema.authenticated = false;
        writeCredentialsStore(name, schema.credentials);
        this.updatedCredentials.next(name);
    }
};
OpenapiAuthService.ctorParameters = () => [
    { type: NgZone },
    { type: HttpClient, decorators: [{ type: Optional }] }
];
OpenapiAuthService = __decorate([
    Injectable(),
    __param(1, Optional()),
    __metadata("design:paramtypes", [NgZone, HttpClient])
], OpenapiAuthService);
function clearCredentialsStore(name) {
    const key = getStorageKey(name);
    window.sessionStorage.removeItem(key);
}
function writeCredentialsStore(name, credentials) {
    const key = getStorageKey(name);
    const value = JSON.stringify(credentials);
    window.sessionStorage.setItem(key, value);
}
function readCredentialsStore(name) {
    const key = getStorageKey(name);
    const value = window.sessionStorage.getItem(key);
    if (typeof value === 'string') {
        try {
            return JSON.parse(value);
        }
        catch (e) {
            console.warn('Failed to parse credentials for ' + name, e);
        }
    }
    return null;
}
function getStorageKey(name) {
    return 'oavCredentials.' + name.replace(/[^-_A-Za-z0-9]/g, '_');
}
function getSecurityInformation(spec) {
    const securitySchemes = [];
    const definitions = spec.securityDefinitions || (spec.components && spec.components.securitySchemes) || {};
    for (const [schemeName, schemeDef] of Object.entries(definitions)) {
        const storedCredentials = readCredentialsStore(schemeName);
        securitySchemes.push({
            name: schemeName,
            securityScheme: schemeDef,
            authenticated: storedCredentials !== null,
            remember: storedCredentials !== null,
            credentials: storedCredentials
        });
    }
    return securitySchemes;
}
function identifyFlows(scheme) {
    const results = [];
    // V2
    if (scheme.flow) {
        let flowName = scheme.flow;
        if (flowName === 'application') {
            flowName = 'clientCredentials';
        }
        if (flowName === 'accessCode') {
            flowName = 'authorizationCode';
        }
        results.push({
            flow: flowName,
            scopes: toScopesInfo(scheme.scopes)
        });
    }
    if (scheme.flows) {
        if (scheme.flows.implicit) {
            results.push({
                flow: 'implicit',
                scopes: toScopesInfo(scheme.flows.implicit.scopes)
            });
        }
        if (scheme.flows.password) {
            results.push({
                flow: 'password',
                scopes: toScopesInfo(scheme.flows.password.scopes)
            });
        }
        if (scheme.flows.clientCredentials) {
            results.push({
                flow: 'clientCredentials',
                scopes: toScopesInfo(scheme.flows.clientCredentials.scopes)
            });
        }
        if (scheme.flows.authorizationCode) {
            results.push({
                flow: 'authorizationCode',
                scopes: toScopesInfo(scheme.flows.authorizationCode.scopes)
            });
        }
    }
    return results;
}
function toScopesInfo(scopes) {
    if (!scopes) {
        return [];
    }
    return Object.keys(scopes).map(name => ({ scope: name, description: scopes[name] }));
}
function getAuthorizationBaseUrl(scheme, flow) {
    if (scheme.type === 'oauth2' && (scheme.flow === 'implicit' || scheme.flow === 'accessCode')) {
        return scheme.authorizationUrl;
    }
    if (scheme.type === 'oauth2' && scheme.flows && scheme.flows[flow]) {
        return scheme.flows[flow].authorizationUrl;
    }
    return null;
}
function getTokenBaseUrl(scheme, flow) {
    if (scheme.type === 'oauth2' && scheme.flow === 'accessCode') {
        return scheme.tokenUrl;
    }
    if (scheme.type === 'oauth2' && scheme.flows && scheme.flows[flow]) {
        return scheme.flows[flow].tokenUrl;
    }
    return null;
}
function buildAuthorizationUrl(scheme, credentials, state) {
    const baseUrl = getAuthorizationBaseUrl(scheme, credentials.flow);
    const params = {
        client_id: credentials.clientId,
        redirect_uri: credentials.redirectUri
    };
    if (credentials.flow === 'implicit') {
        params.response_type = 'token';
        params.nonce = credentials.nonce;
    }
    if (credentials.flow === 'authorizationCode') {
        params.response_type = 'code';
    }
    if (state) {
        params.state = state;
    }
    return baseUrl + '?' + serializeQueryParams(params);
}

let OpenapiViewerService = class OpenapiViewerService {
    constructor(http, authService) {
        this.http = http;
        this.authService = authService;
        this.spec = new BehaviorSubject(null);
        this.tagIndex = new BehaviorSubject([]);
        this.loadErrors = new BehaviorSubject([]);
        this.operationParameterCache = {};
        this.requests = [];
        this.loadingPromise = null;
    }
    loadSpec(spec) {
        return this.resolveSpec({ spec });
    }
    loadSpecByUrl(url) {
        return this.resolveSpec({ url });
    }
    resolveSpec(swaggerOpts) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.loadErrors.value.length) {
                this.loadErrors.next([]);
            }
            if (this.tagIndex.value.length) {
                this.tagIndex.next([]);
            }
            this.loadingPromise = Swagger.resolve({ spec: swaggerOpts.spec, url: swaggerOpts.url });
            const loadingPromise = this.loadingPromise;
            const resolveResult = yield loadingPromise;
            if (loadingPromise !== this.loadingPromise) {
                return null;
            }
            this.spec.next(resolveResult.spec);
            this.tagIndex.next(getTagIndex(resolveResult.spec));
            this.authService.identifySchemes(resolveResult.spec);
            this.operationParameterCache = {};
            if (resolveResult.errors && resolveResult.errors.length) {
                this.loadErrors.next(resolveResult.errors);
                throw new Error('Spec resolve errors');
            }
            return resolveResult.spec;
        });
    }
    createRequest(operationId, parameters, requestBody, requestContentType, responseContentType) {
        // See https://github.com/swagger-api/swagger-js/blob/master/src/execute/index.js#L91
        const securities = {};
        for (const scheme of this.authService.securitySchemes.value) {
            if (scheme.authenticated) {
                securities[scheme.name] = scheme.credentials;
            }
        }
        const params = {
            spec: this.spec.value,
            operationId,
            parameters,
            securities: { authorized: securities },
            requestContentType,
            responseContentType,
            scheme: undefined,
            requestInterceptor: undefined,
            responseInterceptor: undefined,
            contextUrl: undefined,
            userFetch: undefined,
            requestBody,
            server: undefined,
            serverVariables: undefined,
            http: undefined
        };
        return Swagger.buildRequest(params);
    }
    runRequest(pathItem, operationsItem, request) {
        const httpEvents = new ReplaySubject(1);
        const reqInfo = {
            id: randomHex(8),
            pathItem,
            operationsItem,
            request,
            running: true,
            canceled: false,
            sentBytes: 0,
            receivedBytes: 0,
            startTs: new Date(),
            endTs: null,
            httpEvents,
            statusText: null,
            status: 0,
            cancel: null
        };
        this.requests.push(reqInfo);
        const headers = new HttpHeaders(request.headers);
        const sub = this.http
            .request(new HttpRequest(request.method, request.url, request.body, {
            headers,
            responseType: 'blob',
            reportProgress: true
        }))
            .subscribe(status => {
            if (status.type === HttpEventType.ResponseHeader || status.type === HttpEventType.Response) {
                reqInfo.status = status.status;
                reqInfo.statusText = status.statusText;
            }
            if (status.type === HttpEventType.UploadProgress) {
                reqInfo.sentBytes = status.loaded;
            }
            if (status.type === HttpEventType.DownloadProgress) {
                reqInfo.receivedBytes = status.loaded;
            }
            if (status.type === HttpEventType.Response) {
                reqInfo.endTs = new Date();
                reqInfo.running = false;
            }
            console.log('status', status);
            httpEvents.next(status);
        }, err => {
            reqInfo.endTs = new Date();
            reqInfo.running = false;
            httpEvents.error(err);
        }, () => {
            reqInfo.running = false;
            httpEvents.complete();
        });
        reqInfo.cancel = () => {
            sub.unsubscribe();
            reqInfo.canceled = true;
            reqInfo.running = false;
        };
        return reqInfo;
    }
    removeRequest(req) {
        this.requests.splice(this.requests.indexOf(req), 1);
    }
    getRequestsByOperationId(opId) {
        return this.requests.filter(req => req.operationsItem.operation.operationId === opId);
    }
    saveOperationParameters(opId, parameters) {
        this.operationParameterCache[opId] = parameters;
    }
    loadOperationParameters(opId) {
        return this.operationParameterCache[opId];
    }
    resetSpec() {
        this.spec.next(null);
        this.tagIndex.next([]);
        this.loadErrors.next([]);
        this.requests = [];
        this.operationParameterCache = {};
    }
};
OpenapiViewerService.ctorParameters = () => [
    { type: HttpClient },
    { type: OpenapiAuthService }
];
OpenapiViewerService = __decorate([
    Injectable(),
    __metadata("design:paramtypes", [HttpClient, OpenapiAuthService])
], OpenapiViewerService);
function getTagIndex(spec) {
    return (spec.tags || [{ name: 'untagged' }]).map(tag => ({
        tag,
        paths: getPathsByTag(spec, tag.name)
    }));
}
function getPathsByTag(spec, tag) {
    const paths = Object.keys(spec.paths);
    const result = [];
    for (const path of paths) {
        const pathObject = spec.paths[path];
        const tags = getTagsOfPath(pathObject);
        if ((tag !== 'untagged' && tags.includes(tag)) || (tag === 'untagged' && tags.length === 0)) {
            const pathItem = {
                path,
                summary: pathObject.summary,
                description: pathObject.description,
                parameters: pathObject.parameters,
                servers: pathObject.servers,
                operations: null
            };
            pathItem.operations = getOperationsOfPath(pathObject, pathItem.parameters);
            result.push(pathItem);
        }
    }
    return result;
}
function getOperationsOfPath(pathObject, parentParameters) {
    const ops = [];
    for (const method of httpMethods) {
        const operation = pathObject[method];
        if (operation) {
            if (!operation.operationId) {
                operation.operationId = 'op-' + randomHex(8);
            }
            const parameters = [];
            if (parentParameters) {
                parameters.push(...parentParameters);
            }
            if (operation.parameters) {
                parameters.push(...operation.parameters);
            }
            ops.push({
                method,
                operation,
                parameters,
                responseTypes: identifyResponseTypes(operation)
            });
        }
    }
    return ops;
}
function getTagsOfPath(pathObject) {
    const tags = [];
    for (const method of httpMethods) {
        const operation = pathObject[method];
        if (operation && operation.tags) {
            tags.push(...operation.tags);
        }
    }
    return tags;
}
function identifyResponseTypes(operation) {
    if (operation.produces) {
        return operation.produces;
    }
    const types = new Set();
    for (const [status, res] of Object.entries(operation.responses)) {
        if (res.content) {
            for (const contentType of Object.keys(res.content)) {
                types.add(contentType);
            }
        }
    }
    return [...types];
}

let OpenapiViewerComponent = class OpenapiViewerComponent {
    constructor(openApiService) {
        this.openApiService = openApiService;
        this.index = [];
        this.loading = false;
        this.error = null;
        this.loadErrors = [];
        this.destroy = new Subject();
    }
    ngOnInit() {
        this.openApiService.tagIndex.pipe(takeUntil(this.destroy)).subscribe(tagIndex => {
            this.index = tagIndex;
        });
        this.openApiService.loadErrors.pipe(takeUntil(this.destroy)).subscribe(errors => {
            this.loadErrors = errors;
        });
    }
    loadSpec(promise) {
        this.loading = true;
        this.error = null;
        return promise.then(() => {
            this.error = null;
            this.loading = false;
        }, e => {
            this.loading = false;
            this.error = e.message;
            console.error(e);
        });
    }
    ngOnDestroy() {
        this.destroy.complete();
    }
    ngOnChanges(changes) {
        if (changes.spec || changes.specUrl) {
            if (this.spec) {
                this.loadSpec(this.openApiService.loadSpec(this.spec));
            }
            else if (this.specUrl) {
                this.loadSpec(this.openApiService.loadSpecByUrl(this.specUrl));
            }
            else {
                this.openApiService.resetSpec();
            }
        }
    }
};
OpenapiViewerComponent.ctorParameters = () => [
    { type: OpenapiViewerService }
];
__decorate([
    Input(),
    __metadata("design:type", Object)
], OpenapiViewerComponent.prototype, "spec", void 0);
__decorate([
    Input(),
    __metadata("design:type", String)
], OpenapiViewerComponent.prototype, "specUrl", void 0);
OpenapiViewerComponent = __decorate([
    Component({
        selector: 'oav-openapi-viewer',
        template: "<div class=\"oav-layout\">\n  <header class=\"oav-header\">\n    <ng-content></ng-content>\n  </header>\n  <div class=\"oav-menu-header\">\n    <a href=\"javascript:\" class=\"oav-menu-toggle\" oavOpenable #openMenu=\"openable\"></a>\n  </div>\n  <div class=\"oav-content-container\">\n    <div *ngIf=\"loading\">\n      <span class=\"loading\"></span>\n      Loading\n    </div>\n\n    <div *ngIf=\"!loading && !index.length\">\n      No Tags found\n    </div>\n\n    <div *ngIf=\"!loading && error\">\n      <h3>Error</h3>\n      <pre>{{ error.toString() }}</pre>\n      <pre *ngFor=\"let err of loadErrors\">{{ err.toString() }}</pre>\n    </div>\n\n    <ng-container *ngIf=\"!loading && !error && index.length\">\n      <aside class=\"oav-index-container\" [class.open]=\"openMenu.open\">\n        <oav-index-nav [index]=\"index\"></oav-index-nav>\n      </aside>\n      <section class=\"oav-view-container\" [class.open]=\"openMenu.open\">\n        <router-outlet></router-outlet>\n      </section>\n    </ng-container>\n  </div>\n</div>\n",
        providers: [OpenapiAuthService, OpenapiViewerService],
        encapsulation: ViewEncapsulation.None,
        styles: ["@charset \"UTF-8\";.oav-layout{height:100vh;display:-webkit-box;display:flex;-webkit-box-orient:vertical;-webkit-box-direction:normal;flex-direction:column}.oav-layout .oav-menu-header{border-bottom:1px solid #ccc}.oav-layout .oav-menu-header .oav-menu-toggle{display:inline-block;color:#000;padding:.5rem;text-decoration:none;font-weight:700;font-size:1.2rem}.oav-layout .oav-menu-header .oav-menu-toggle::before{content:\"\u2630\"}.oav-layout .oav-content-container{-webkit-box-flex:1;flex:1;display:-webkit-box;display:flex;min-height:0;position:relative}.oav-layout .oav-content-container .oav-index-container{width:18rem;margin-left:-18rem;overflow:auto;background:#f6f6fb;-webkit-transition:margin-left .1s ease-out;transition:margin-left .1s ease-out}.oav-layout .oav-content-container .oav-index-container.open{margin-left:0}.oav-layout .oav-content-container .oav-view-container{-webkit-box-flex:1;flex:1;overflow:scroll;margin-right:0;-webkit-transition:margin-right .1s ease-out;transition:margin-right .1s ease-out}.oav-layout .oav-content-container .oav-view-container.open{margin-right:-18rem}.oav-content{margin:1rem}.oav-content h1{font-size:1.5em}.oav-content h3{font-size:1.15em}.oav-content h4{font-size:1em}.oav-content a{color:#2681ac}.oav-content a:hover{color:#31b1ff}.oav-content .deprecated{text-decoration:line-through}.oav-content .oav-path{font-family:monospace}.oav-content .oav-summary{font-weight:400}.oav-content .send-btn{color:#fff;background-color:#2681ac;font-size:1rem;padding:.5rem 1rem;border:0;margin:.2rem 0}.oav-content .send-btn:hover{background-color:#31b1ff}.oav-content .send-btn:disabled{background-color:#ccc}.oav-content pre{white-space:pre-wrap}.oav-content .oav-warning{color:#fb9200}.oav-content .ace_editor{border:1px solid #ccc}oav-json-schema .oav-json-schema-code{font-family:monospace}oav-json-schema .oav-json-schema-toggle{font-weight:700;text-decoration:none}oav-json-schema .json-schema-property{display:-webkit-box;display:flex;-webkit-box-align:start;align-items:flex-start}oav-json-schema .oav-json-schema-name.oav-json-schema-required::after{content:\"*\";color:#ac0003;padding-left:.1em}oav-json-schema .oav-json-schema-description{font-size:.9rem;margin-left:.5rem}oav-json-schema .oav-json-schema-description p{margin-top:0}oav-json-schema .oav-json-schema-description p:last-child{margin-bottom:0}oav-json-schema .oav-json-schema-type{margin-left:.2rem}oav-json-schema .oav-json-schema-type-name{color:#8a8a8f}.oav-method{width:5em;display:inline-block;color:#fff;text-align:center;padding:.2em;font-size:.7em}.oav-method.oav-method-get{background-color:#2681ac}.oav-method.oav-method-post{background-color:#25ac20}.oav-method.oav-method-put{background-color:#ac700e}.oav-method.oav-method-delete{background-color:#ac0003}.oav-method.oav-method-options{background-color:#ac03a6}.oav-method.oav-method-head{background-color:#87aaac}.oav-method.oav-method-trace{background-color:#87ac89}a.oav-header-link{color:#000;text-decoration:none}a.oav-header-link:hover{color:#2681ac}.oav-method-bg.method-get{background-color:rgba(38,129,172,.1)}.oav-method-bg.method-post{background-color:rgba(37,172,32,.1)}.oav-method-bg.method-put{background-color:rgba(172,112,14,.1)}.oav-method-bg.method-delete{background-color:rgba(172,0,3,.1)}.oav-method-bg.method-options{background-color:rgba(172,3,166,.1)}.oav-method-bg.method-head{background-color:rgba(135,170,172,.1)}.oav-method-bg.method-trace{background-color:rgba(135,172,137,.1)}oav-method-header h3{cursor:pointer;display:-webkit-box;display:flex;-webkit-box-align:center;align-items:center;padding:.5rem;margin:0}oav-method-header h3 .mainlink{text-decoration:inherit;color:inherit;cursor:pointer;font-size:1rem}oav-method-header h3 .method-get{background-color:#2681ac}oav-method-header h3 .method-post{background-color:#25ac20}oav-method-header h3 .method-put{background-color:#ac700e}oav-method-header h3 .method-delete{background-color:#ac0003}oav-method-header h3 .method-options{background-color:#ac03a6}oav-method-header h3 .method-head{background-color:#87aaac}oav-method-header h3 .method-trace{background-color:#87ac89}oav-method-header h3:hover{background-color:rgba(135,172,137,.2)}oav-method-header h3 .path{font-family:monospace;font-size:1.2rem}oav-method-header h3 .summary{padding:0 10px;font-size:.8rem;font-weight:400;-webkit-box-flex:1;flex:1}oav-method-header h3 .summary.extra-link{-webkit-box-flex:0;flex:none}.openapi-operation{margin:.5rem 0}.openapi-operation .oav-operation-details{display:none}.openapi-operation.open{padding-bottom:1rem}.openapi-operation.open .oav-operation-details{display:block}.openapi-operation.deprecated h3 .mainlink{text-decoration:line-through}.oav-operation-details{margin:.5rem 0;padding:0 2rem}.oav-operation-details .send-btn{color:#fff;background-color:#2681ac;font-size:1rem;padding:.5rem 1rem;border:0;margin:.2rem 0}.oav-operation-details .send-btn:hover{background-color:#31b1ff}.oav-operation-details pre{white-space:pre-wrap}.oav-operation-details h4{background-color:rgba(255,255,255,.6);padding:.5rem 2rem;margin:.5rem -2rem}.oav-operation-layout{display:-webkit-box;display:flex}.oav-operation-layout .oav-operation-details{-webkit-box-flex:1;flex:1}.oav-operation-layout .oav-operation-requests{min-width:30rem;margin-left:1rem}@media (max-width:80rem){.oav-operation-layout{display:block}.oav-operation-layout .oav-operation-requests{margin-left:0}}.toggle-button{border:none;background:0 0;outline:0}.toggle-button:before{font-size:1rem;font-weight:600;content:\"\u3009\";display:inline-block;-webkit-transform-origin:25% 50%;transform-origin:25% 50%;-webkit-transition:-webkit-transform .2s ease-out;transition:transform .2s ease-out;transition:transform .2s ease-out,-webkit-transform .2s ease-out}.toggle-button.open:before{-webkit-transform:rotate(90deg);transform:rotate(90deg)}.toggle-button.full-rotate.open:before{-webkit-transform:rotate(180deg);transform:rotate(180deg)}.download-btn{color:#fff;background-color:#2681ac;font-size:1rem;padding:.5rem 1rem;border:0;margin:.2rem 0}.download-btn:hover{background-color:#31b1ff}.dismiss-btn{color:#fff;background-color:#ac0003;font-size:1rem;padding:.5rem 1rem;border:0;margin:.2rem 0}.dismiss-btn:hover{background-color:#d60003}.oav-parameter{border-bottom:1px solid #ccc;padding:.2rem 0;box-sizing:border-box}.oav-parameter .input,.oav-parameter .name{margin:.2rem 0;padding-right:.5rem}.oav-parameter .input input:not([type=checkbox]),.oav-parameter .input select,.oav-parameter .input textarea{width:100%;padding:.2rem;font-size:1rem;font-family:monospace;box-sizing:border-box}.oav-parameter .input textarea{min-height:10em}.oav-parameter .description{margin:.2rem 0}.oav-parameter .description p{margin-top:0}.oav-parameter .description p:last-child{margin-bottom:0}.oav-parameter .required{color:#ac0003}.oav-parameter .format,.oav-parameter .param-in{color:#828282;font-size:.8rem}.multi-item-entry{display:-webkit-box;display:flex;margin-bottom:.2rem}.multi-item-entry input{padding:.2rem;font-size:1rem;font-family:monospace;box-sizing:border-box}.multi-item-entry button{color:#fff;background-color:#a7a7ac;font-size:1rem;padding:.2rem 1rem;border:0;min-width:2.6rem;margin-left:.2rem}.multi-item-entry button:hover{background-color:#d4d4d9}.oav-add-btn{color:#fff;background-color:#a7a7ac;font-size:1rem;padding:.2rem 1rem;border:0;min-width:2.6rem}.oav-add-btn:hover{background-color:#d4d4d9}.oav-btn-container{text-align:right}.oav-divided-container .main{-webkit-box-flex:1;flex:1}.oav-auth-status{font-family:monospace;cursor:pointer}.oav-auth-status::before{content:\" \"}.oav-auth-status.required::before{content:\"\uD83D\uDD12\uFE0E\";color:#8a8a8f}.oav-auth-status.ok::before{content:\"\uD83D\uDD13\uFE0E\";color:#25ac20}.oav-request{background:#f1f1f1;margin:.5rem 0}.oav-request.method-get{background-color:rgba(38,129,172,.1)}.oav-request.method-get h3:hover{background-color:rgba(38,129,172,.2)}.oav-request.method-post{background-color:rgba(37,172,32,.1)}.oav-request.method-post h3:hover{background-color:rgba(37,172,32,.2)}.oav-request.method-put{background-color:rgba(172,112,14,.1)}.oav-request.method-put h3:hover{background-color:rgba(172,112,14,.2)}.oav-request.method-delete{background-color:rgba(172,0,3,.1)}.oav-request.method-delete h3:hover{background-color:rgba(172,0,3,.2)}.oav-request.method-options{background-color:rgba(172,3,166,.1)}.oav-request.method-options h3:hover{background-color:rgba(172,3,166,.2)}.oav-request.method-head{background-color:rgba(135,170,172,.1)}.oav-request.method-head h3:hover{background-color:rgba(135,170,172,.2)}.oav-request.method-trace{background-color:rgba(135,172,137,.1)}.oav-request.method-trace h3:hover{background-color:rgba(135,172,137,.2)}.oav-request h4{background-color:rgba(255,255,255,.6);padding:.5rem 2rem;margin:.5rem -2rem}.oav-request .content{padding:1rem}.oav-request textarea{font-family:monospace;white-space:pre-wrap;box-sizing:border-box;background:#fff;width:100%;min-height:5rem}.oav-request .status-200,.oav-request .status-201,.oav-request .status-202,.oav-request .status-204{color:#25ac20}.oav-request .status-400,.oav-request .status-401,.oav-request .status-402,.oav-request .status-403,.oav-request .status-404,.oav-request .status-405,.oav-request .status-406,.oav-request .status-500,.oav-request .status-501,.oav-request .status-503,.oav-request .status-504,.oav-request .status-505{color:#ac0003}.oav-tab-nav{border-bottom:1px solid #ccc}.oav-tab-nav a{color:#000;display:inline-block;text-decoration:none;padding:.5rem 1rem}.oav-tab-nav a.active,.oav-tab-nav a:hover{color:#2681ac}.oav-tab-nav a.active{padding-bottom:.3rem;border-bottom:.2rem solid #2681ac}.oav-index-nav{padding:0 0 1rem;font-size:.8rem}.oav-index-nav .deprecated{text-decoration:line-through}.oav-index-nav header a{color:#000;text-decoration:none;padding:.2em 1rem;font-size:1rem;display:-webkit-box;display:flex;-webkit-box-pack:justify;justify-content:space-between;font-weight:700}.oav-index-nav header a:hover{color:#2681ac;background:#fff}.oav-index-nav ol{padding-left:0}.oav-index-nav li{padding:0;display:block}.oav-index-nav li oav-method{margin-right:.5em}.oav-index-nav li a{color:#000;text-decoration:none;padding:.2em .2em .2em 1rem;box-sizing:border-box;display:block;position:relative}.oav-index-nav li a .label{display:-webkit-box;display:flex}.oav-index-nav li a .info-overlay{padding:.2em .2em .2em 1rem;display:none;position:absolute;background:#fff;top:0;left:0;white-space:nowrap}.oav-index-nav li a:hover{background:#fff;color:#2681ac}.oav-index-nav li a:hover .info-overlay{display:-webkit-box;display:flex}.oav-index-nav li a:hover .label{visibility:hidden}.oav-index-nav li a.active,.oav-index-nav li a:active{background:#fff;font-weight:700;color:#2681ac}.oav-index-nav .oav-bottom-nav,.oav-index-nav .oav-top-nav{border-bottom:1px solid #ccc;margin:.5rem 0;padding:.5rem 0}.oav-index-nav .oav-bottom-nav a,.oav-index-nav .oav-top-nav a{font-size:1rem;padding-left:1rem}.oav-index-nav .oav-bottom-nav{border-bottom:0;border-top:1px solid #ccc}.m-0{margin:0}.my-0{margin-top:0;margin-bottom:0}.my-3{margin-top:1rem;margin-bottom:1rem}.mt-0{margin-top:0}.ml-auto{margin-left:auto}.ml-0{margin-left:0!important}.ml-1{margin-left:.25rem}.ml-2{margin-left:.5rem}.ml-3{margin-left:1rem}.font-size-tiny{font-size:.5rem}.font-size-smaller{font-size:.8rem}.font-size-1{font-size:1rem}.font-size-2{font-size:2rem}.font-size-3{font-size:3rem}.d-flex{display:-webkit-box;display:flex}.align-items-center{-webkit-box-align:center;align-items:center}.flex-grow-1{-webkit-box-flex:1;flex:1}.justify-content-between{-webkit-box-pack:justify;justify-content:space-between}.normal-text{font-size:1rem;font-weight:400}.oav-error{color:#ac0003}.openable{cursor:pointer}.loading:before{content:\"\u21BB\";display:inline-block;-webkit-animation:2s linear infinite spin-loader;animation:2s linear infinite spin-loader}oav-tooltip{display:block;background:#fff;border:1px solid #ccc;padding:1rem;box-shadow:0 0 3px #ccc}@-webkit-keyframes spin-loader{to{-webkit-transform:rotate(360deg);transform:rotate(360deg)}}@keyframes spin-loader{to{-webkit-transform:rotate(360deg);transform:rotate(360deg)}}.openapi-response{border-bottom:1px solid #ccc;padding:.2rem 0;box-sizing:border-box}.openapi-response .status-code{margin:.2rem 0;padding-right:.5rem}.openapi-response .description{margin:.2rem 0}@media (min-width:60rem){.d-md-flex{display:-webkit-box;display:flex}.oav-layout .oav-menu-header{display:none}.oav-layout .oav-content-container .oav-index-container{margin-left:0}.oav-layout .oav-content-container .oav-view-container.open{margin-right:0}.oav-parameter{display:-webkit-box;display:flex}.oav-parameter .name{width:15rem}.oav-parameter .description,.oav-parameter .input{width:50%}.openapi-response{display:-webkit-box;display:flex}.openapi-response .status-code{width:15rem}.openapi-response .description{min-width:30rem}.oav-divided-container{display:-webkit-box;display:flex}}"]
    }),
    __metadata("design:paramtypes", [OpenapiViewerService])
], OpenapiViewerComponent);

class OavSettings {
    constructor(overwriteSettings) {
        /**
         * Which value of an operation is shown in the index navigation, in which order.
         * The first one is displayed if available, otherwise the nex one will be  chosen until a value is available.
         */
        this.indexPrimaryLabel = ['summary', 'operationId', 'path'];
        /**
         * Which value is shown in the index navigation on hover.
         */
        this.indexHoverLabel = ['path'];
        /**
         * Which value of an operation is shown in the overview navigation, in which order.
         * The first one is displayed if available, otherwise the nex one will be  chosen until a value is available.
         */
        this.overviewPrimaryLabel = ['summary', 'operationId'];
        /**
         * Whether to show the option to display the raw operation definition on the operation page.
         */
        this.showRawOperationDefinition = false;
        /**
         * Show raw model definitions
         */
        this.showRawModelDefinition = false;
        /**
         * Whether authentication is enabled.
         * - Status is shown at operations
         * - Authentication page is shown, where a user can enter credentials
         */
        this.enableAuthentication = true;
        /**
         * Show the models overview page
         */
        this.enableModelsOverview = true;
        if (overwriteSettings) {
            Object.assign(this, overwriteSettings);
        }
    }
}
const defaultOavSettings = new OavSettings();

let OperationComponent = class OperationComponent {
    constructor(openApiService, oavSettings, authService, cd) {
        this.openApiService = openApiService;
        this.oavSettings = oavSettings;
        this.authService = authService;
        this.cd = cd;
        this.responseType = new FormControl('application/json');
        this.requests = [];
        this.openRequest = null;
        this.showRawOperationDefinition = false;
        this.showAuthentication = true;
        if (!this.oavSettings) {
            this.oavSettings = defaultOavSettings;
        }
        this.showRawOperationDefinition = this.oavSettings.showRawOperationDefinition;
        this.showAuthentication = this.oavSettings.enableAuthentication;
    }
    get operation() {
        return this.operationItem.operation;
    }
    ngOnChanges(changes) {
        if (changes.operationItem && this.operationItem) {
            if (changes.operationItem.previousValue && this.formGroup) {
                const prevOperation = changes.operationItem.previousValue;
                this.openApiService.saveOperationParameters(prevOperation.operation.operationId, this.formGroup.value);
            }
            this.requests = this.openApiService.getRequestsByOperationId(this.operationItem.operation.operationId).reverse();
            this.formGroup = new FormGroup({ __responseType: this.responseType });
            this.responseType.patchValue(this.operationItem.responseTypes[0] || 'application/json');
            this.authStatus = this.authService.getAuthStatus(this.operationItem.operation.security);
            const savedParams = this.openApiService.loadOperationParameters(this.operation.operationId);
            if (savedParams) {
                this.cd.detectChanges();
                this.formGroup.patchValue(savedParams);
            }
        }
    }
    ngOnDestroy() {
        if (this.operationItem) {
            this.openApiService.saveOperationParameters(this.operation.operationId, this.formGroup.value);
        }
    }
    send() {
        try {
            const parameters = Object.assign({}, this.formGroup.value);
            let requestBody;
            let contentType = '';
            if (parameters.requestBody) {
                contentType = parameters.requestBody.contentType;
                requestBody = Object.assign({}, parameters.requestBody);
                delete requestBody.contentType;
                delete parameters.requestBody;
            }
            const req = this.openApiService.createRequest(this.operationItem.operation.operationId, parameters, requestBody, contentType, this.responseType.value);
            const reqInfo = this.openApiService.runRequest(this.pathItem, this.operationItem, req);
            this.requests.unshift(reqInfo);
            this.openRequest = reqInfo;
        }
        catch (e) {
            console.warn('Create request error', e);
        }
    }
};
OperationComponent.ctorParameters = () => [
    { type: OpenapiViewerService },
    { type: OavSettings, decorators: [{ type: Optional }] },
    { type: OpenapiAuthService },
    { type: ChangeDetectorRef }
];
__decorate([
    Input(),
    __metadata("design:type", String)
], OperationComponent.prototype, "tag", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], OperationComponent.prototype, "pathItem", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], OperationComponent.prototype, "operationItem", void 0);
OperationComponent = __decorate([
    Component({
        selector: 'oav-operation',
        template: "<h1>\n  <oav-method [method]=\"operationItem.method\"></oav-method>\n  <oav-auth-status [operation]=\"operationItem.operation\"></oav-auth-status>\n  <span class=\"oav-path ml-1\" [class.deprecated]=\"operation.deprecated\">{{ pathItem.path }}</span>\n  <span class=\"oav-summary font-size-smaller ml-2\">{{ operation.summary }}</span>\n</h1>\n\n<p *ngIf=\"operationItem.operation.description\">\n  <markdown [data]=\"operationItem.operation.description\"></markdown>\n</p>\n\n<form (submit)=\"send()\" [formGroup]=\"formGroup\">\n  <ng-container *ngIf=\"operationItem.parameters?.length\">\n    <h3>Parameters</h3>\n    <oav-parameter *ngFor=\"let param of operationItem.parameters\" [parameter]=\"param\" [formGroup]=\"formGroup\"></oav-parameter>\n  </ng-container>\n  <ng-container *ngIf=\"operationItem.operation.requestBody\">\n    <oav-request-body-input [requestBody]=\"operationItem.operation.requestBody\" [addToFormGroup]=\"formGroup\"></oav-request-body-input>\n  </ng-container>\n  <div *ngIf=\"operationItem.responseTypes?.length > 1\">\n    <div class=\"oav-parameter\">\n      <div class=\"name\">\n        <div><strong>Response type</strong>:</div>\n        <div class=\"param-in\">\n          Accept header\n        </div>\n      </div>\n      <div class=\"input\">\n        <select [formControl]=\"responseType\">\n          <option *ngFor=\"let resType of operationItem.responseTypes\" [value]=\"resType\">{{ resType }}</option>\n        </select>\n      </div>\n      <div class=\"description\"></div>\n    </div>\n  </div>\n  <button class=\"send-btn\" [disabled]=\"formGroup.invalid\">Send request</button>\n</form>\n\n<h3 oavOpenable #responseOpenable=\"openable\" class=\"toggle-button\">Responses</h3>\n\n<oav-responses\n  *ngIf=\"responseOpenable.open\"\n  [responses]=\"operationItem.operation.responses\"\n  [responseType]=\"responseType.value\"\n></oav-responses>\n\n<div class=\"oav-operation-requests\" [hidden]=\"!requests.length\">\n  <h3>Requests</h3>\n  <oav-requests-list [requests]=\"requests\" [openRequest]=\"openRequest\"></oav-requests-list>\n</div>\n\n<ng-container *ngIf=\"showRawOperationDefinition\">\n  <h3 oavOpenable #rawOpenable=\"openable\" class=\"toggle-button\">Raw operation definition</h3>\n  <oav-code-input *ngIf=\"rawOpenable.open\" mode=\"json\" [ngModel]=\"operationItem.operation | json\" readonly></oav-code-input>\n</ng-container>\n"
    }),
    __param(1, Optional()),
    __metadata("design:paramtypes", [OpenapiViewerService,
        OavSettings,
        OpenapiAuthService,
        ChangeDetectorRef])
], OperationComponent);

const primitiveTypes = ['string', 'integer', 'number', 'boolean'];
const ɵ0$1 = (param) => param.type === 'string' && param.items && param.items.enum, ɵ1$1 = (param) => schemaType(param) === 'string' && param.schema.enum, ɵ2$1 = (param, mediaType) => getExample(param, mediaType), ɵ3$1 = (param) => param.type === 'string', ɵ4$1 = (param) => schemaType(param) === 'string', ɵ5$1 = (param, mediaType) => getExample(param, mediaType), ɵ6$1 = (param) => param.type === 'integer', ɵ7$1 = (param) => schemaType(param) === 'integer', ɵ8$1 = (param, mediaType) => getExample(param, mediaType), ɵ9$1 = (param) => param.type === 'object', ɵ10$1 = (param) => schemaType(param) === 'object' || (param.schema && param.schema.properties), ɵ11$1 = (param, mediaType) => getExample(param, mediaType), ɵ12$1 = (param) => param.type === 'array' && param.items && param.items.enum, ɵ13$1 = (param) => schemaType(param) === 'array' && param.schema.enum, ɵ14$1 = (param, mediaType) => getExample(param, mediaType), ɵ15 = (param) => param.type === 'array' && primitiveTypes.includes(itemsType(param)), ɵ16 = (param) => schemaType(param) === 'array' && primitiveTypes.includes(itemsType(param.schema)), ɵ17 = (param, mediaType) => getExample(param, mediaType), ɵ18 = (param) => param.type === 'array', ɵ19 = (param) => schemaType(param) === 'array', ɵ20 = (param, mediaType) => getExample(param, mediaType), ɵ21 = (param) => param.type === 'file', ɵ22 = (param) => false, ɵ23 = (param, mediaType) => '', ɵ24 = (param) => true, ɵ25 = (param) => true, ɵ26 = (param, mediaType) => '';
const displayModes = [
    {
        mode: 'textSelect',
        checkV2: ɵ0$1,
        checkV3: ɵ1$1,
        getDefault: ɵ2$1
    },
    {
        mode: 'text',
        checkV2: ɵ3$1,
        checkV3: ɵ4$1,
        getDefault: ɵ5$1
    },
    {
        mode: 'integer',
        checkV2: ɵ6$1,
        checkV3: ɵ7$1,
        getDefault: ɵ8$1
    },
    {
        mode: 'object',
        checkV2: ɵ9$1,
        checkV3: ɵ10$1,
        getDefault: ɵ11$1
    },
    {
        mode: 'arrayWithSelection',
        checkV2: ɵ12$1,
        checkV3: ɵ13$1,
        getDefault: ɵ14$1
    },
    {
        mode: 'arrayWithPrimitive',
        checkV2: ɵ15,
        checkV3: ɵ16,
        getDefault: ɵ17
    },
    {
        mode: 'array',
        checkV2: ɵ18,
        checkV3: ɵ19,
        getDefault: ɵ20
    },
    {
        mode: 'file',
        checkV2: ɵ21,
        checkV3: ɵ22,
        getDefault: ɵ23
    },
    {
        mode: 'any',
        checkV2: ɵ24,
        checkV3: ɵ25,
        getDefault: ɵ26
    }
];
function schemaType(param) {
    const schema = param && 'schema' in param ? param.schema : null;
    return schema && schema.type;
}
function itemsType(obj) {
    if (obj && Array.isArray(obj.items)) {
        return obj.items[0].type;
    }
    else if (obj && obj.items) {
        return obj.items.type;
    }
    console.warn('items type not found', obj);
}
function getExample(param, mediaType) {
    if (param.example !== undefined) {
        return param.example;
    }
    if (param.examples !== undefined && param.examples[mediaType] && 'value' in param.examples[mediaType]) {
        return param.examples[mediaType].value;
    }
    if (param.schema) {
        return exampleFromSchema(param.schema);
    }
    if (param.type === 'array' && param.items && 'default' in param.items) {
        return [param.items.default];
    }
    if (isPrimitiveType(param.type) && param.items && 'default' in param.items) {
        return param.items.default;
    }
    return exampleFromSchema(param);
}
function getDisplayMode(param) {
    for (const dm of displayModes) {
        if ('type' in param && dm.checkV2(param)) {
            return dm.mode;
        }
        else if (dm.checkV3(param)) {
            return dm.mode;
        }
    }
    return 'any';
}
function getExampleValue(displayMode, param, mediaType) {
    return displayModes.find(dm => dm.mode === displayMode).getDefault(param, mediaType);
}
function isPrimitiveType(type) {
    return ['number', 'boolean', 'string', 'integer'].includes(type);
}

let ParameterComponent = class ParameterComponent {
    constructor() {
        this.mediaType = '';
        this.displayMode = 'any';
        this.enum = null;
    }
    ngOnChanges(changes) {
        this.displayMode = getDisplayMode(this.parameter);
        this.value = getExampleValue(this.displayMode, this.parameter, this.mediaType);
        if (this.displayMode === 'object' || this.displayMode === 'array') {
            this.value = JSON.stringify(this.value, null, 2);
        }
        const validators = [];
        if (this.parameter.required) {
            validators.push(Validators.required);
        }
        this.control = new FormControl(this.value, validators);
        if (this.formGroup) {
            this.formGroup.addControl(this.parameter.name, this.control);
        }
        this.enum =
            (this.parameter.items && this.parameter.items.enum) || (this.parameter.schema && this.parameter.schema.enum);
    }
    ngOnDestroy() {
        if (this.formGroup) {
            this.formGroup.removeControl(this.parameter.name);
        }
    }
};
__decorate([
    Input(),
    __metadata("design:type", FormGroup)
], ParameterComponent.prototype, "formGroup", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], ParameterComponent.prototype, "parameter", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], ParameterComponent.prototype, "mediaType", void 0);
ParameterComponent = __decorate([
    Component({
        selector: 'oav-parameter',
        template: "<div class=\"oav-parameter\" [formGroup]=\"formGroup\">\n  <div class=\"name\">\n    <div>\n      <strong>{{ parameter.name }}</strong\n      ><span class=\"required\" *ngIf=\"parameter.required\" title=\"required\">*</span>:\n      <span class=\"type\">{{ parameter.type }}</span>\n    </div>\n    <div class=\"param-in\">\n      {{ parameter.customIn ? parameter.customIn : parameter.in }}\n      <span class=\"format\" *ngIf=\"parameter.format\">({{ parameter.format }})</span>\n    </div>\n  </div>\n  <div class=\"input\">\n    <ng-container *ngIf=\"displayMode === 'textSelect'\">\n      <select [formControlName]=\"parameter.name\" [name]=\"parameter.name\">\n        <option *ngFor=\"let opt of enum\" [value]=\"opt\">{{ opt }}</option>\n      </select>\n    </ng-container>\n\n    <ng-container *ngIf=\"displayMode === 'text'\">\n      <input [formControlName]=\"parameter.name\" [placeholder]=\"parameter.name\" [name]=\"parameter.name\" />\n    </ng-container>\n\n    <ng-container *ngIf=\"displayMode === 'integer'\">\n      <input type=\"number\" [formControlName]=\"parameter.name\" [placeholder]=\"parameter.name\" [name]=\"parameter.name\" />\n    </ng-container>\n\n    <ng-container *ngIf=\"displayMode === 'number'\">\n      <input type=\"number\" [formControlName]=\"parameter.name\" [placeholder]=\"parameter.name\" [name]=\"parameter.name\" />\n    </ng-container>\n\n    <ng-container *ngIf=\"displayMode === 'file'\">\n      <input type=\"file\" [formControlName]=\"parameter.name\" [placeholder]=\"parameter.name\" [name]=\"parameter.name\" />\n    </ng-container>\n\n    <ng-container *ngIf=\"displayMode === 'object' || displayMode === 'array'\">\n      <oav-code-input [formControlName]=\"parameter.name\" mode=\"json\" [schema]=\"parameter.schema\"></oav-code-input>\n    </ng-container>\n\n    <ng-container *ngIf=\"displayMode === 'arrayWithSelection'\">\n      <select [formControlName]=\"parameter.name\" [name]=\"parameter.name\" multiple>\n        <option *ngFor=\"let opt of enum\" [ngValue]=\"opt\">{{ opt }}</option>\n      </select>\n    </ng-container>\n\n    <ng-container *ngIf=\"displayMode === 'arrayWithPrimitive'\">\n      <oav-multi-items-input [formControlName]=\"parameter.name\"></oav-multi-items-input>\n    </ng-container>\n  </div>\n  <div class=\"description\">\n    <markdown [data]=\"parameter.description\"></markdown>\n\n    <ng-container *ngIf=\"displayMode === 'object' || displayMode === 'array'\">\n      <oav-json-schema [schema]=\"parameter.schema\"></oav-json-schema>\n    </ng-container>\n  </div>\n</div>\n"
    }),
    __metadata("design:paramtypes", [])
], ParameterComponent);

let ResponsesComponent = class ResponsesComponent {
    constructor() {
        this.items = [];
        this.mode = 'text';
    }
    ngOnChanges(changes) {
        if (!this.responses) {
            return;
        }
        if (this.responseType && this.responseType.startsWith('application/json')) {
            this.mode = 'json';
        }
        else if (this.responseType && this.responseType.startsWith('application/xml')) {
            this.mode = 'xml';
        }
        else {
            this.mode = 'text';
        }
        this.items = getExamplesByContentType(this.responses, this.responseType);
    }
};
__decorate([
    Input(),
    __metadata("design:type", Object)
], ResponsesComponent.prototype, "responses", void 0);
__decorate([
    Input(),
    __metadata("design:type", String)
], ResponsesComponent.prototype, "responseType", void 0);
ResponsesComponent = __decorate([
    Component({
        selector: 'oav-responses',
        template: "<oav-tab-nav>\n  <ng-container *ngFor=\"let status of items\">\n    <div *oavTabNavItem=\"status.statusCode\">\n      <markdown *ngIf=\"status.description\" [data]=\"status.description\"></markdown>\n      <ng-container *ngIf=\"status.headers?.length\">\n        <h4>Example Headers</h4>\n        <div class=\"my-3\">\n          <div class=\"oav-parameter\" *ngFor=\"let header of status.headers\">\n            <div class=\"name\">{{ header.name }}:</div>\n            <div class=\"input\">{{ header.value }}</div>\n            <div class=\"description\">\n              <markdown [data]=\"header.description\"></markdown>\n            </div>\n          </div>\n        </div>\n      </ng-container>\n      <ng-container *ngIf=\"status.contents?.length === 1\">\n        <h4>Example Body</h4>\n        <div class=\"oav-divided-container\">\n          <div class=\"flex-grow-1\">\n            <oav-code-input readonly [ngModel]=\"status.contents[0].value\" [mode]=\"mode\"></oav-code-input>\n          </div>\n          <div *ngIf=\"status.contents[0].schema\" class=\"flex-grow-1 ml-3\">\n            <oav-json-schema [schema]=\"status.contents[0].schema\"></oav-json-schema>\n          </div>\n        </div>\n      </ng-container>\n      <ng-container *ngIf=\"status.contents?.length > 1\">\n        <oav-tab-nav>\n          <ng-container *ngFor=\"let content of status.contents\">\n            <div *oavTabNavItem=\"content.summary || content.name\">\n              <markdown *ngIf=\"content.description\" [data]=\"content.description\"></markdown>\n              <a *ngIf=\"content.externalValueUrl\" [href]=\"content.externalValueUrl\" target=\"_blank\">Example</a>\n              <div class=\"oav-divided-container\" *ngIf=\"!content.externalValueUrl\">\n                <div class=\"flex-grow-1\">\n                  <oav-code-input readonly [ngModel]=\"content.value\" [mode]=\"mode\" [maxLines]=\"20\"></oav-code-input>\n                </div>\n                <div *ngIf=\"content.schema\" class=\"flex-grow-1 ml-3\">\n                  <oav-json-schema [schema]=\"content.schema\"></oav-json-schema>\n                </div>\n              </div>\n            </div>\n          </ng-container>\n        </oav-tab-nav>\n      </ng-container>\n    </div>\n  </ng-container>\n</oav-tab-nav>\n"
    }),
    __metadata("design:paramtypes", [])
], ResponsesComponent);
function getExamplesByContentType(responses, contentType) {
    return Object.keys(responses).map(statusCode => {
        const res = responses[statusCode];
        return {
            statusCode,
            description: res.description,
            headers: getExampleHeaders(res.headers),
            contents: getExampleContents(res, contentType)
        };
    });
}
function getExampleContents(res, contentType) {
    if (res.schema) {
        return [
            {
                value: JSON.stringify(exampleFromSchema(res.schema), null, 2),
                schema: res.schema
            }
        ];
    }
    if (!res.content) {
        return [];
    }
    let content = res.content[contentType];
    if (!content) {
        const mediaType = Object.keys(res.content).find(mt => getStarMatcher(mt).test(contentType));
        if (mediaType) {
            content = res.content[mediaType];
        }
    }
    if (!content) {
        return [];
    }
    if (content.example && contentType.includes('json')) {
        return [{ value: getJsonString(content.example) }];
    }
    if (content.examples) {
        return Object.keys(content.examples).map(exName => {
            const example = content.examples[exName];
            return {
                name: exName,
                summary: example.summary,
                description: example.description,
                value: getJsonString(example.value),
                externalValueUrl: example.externalValue
            };
        });
    }
    if (content.schema) {
        return [
            {
                value: JSON.stringify(exampleFromSchema(content.schema), null, 2),
                schema: content.schema
            }
        ];
    }
    return [];
}
function getExampleHeaders(headers) {
    if (!headers) {
        return [];
    }
    return Object.keys(headers).map(headerName => {
        const header = headers[headerName];
        return {
            name: headerName,
            description: header.description,
            deprecated: header.deprecated,
            value: getHeaderExampleValue(header)
        };
    });
}
function getHeaderExampleValue(headerObject) {
    const displayMode = getDisplayMode(headerObject);
    return getExampleValue(displayMode, headerObject, null);
}

var MultiItemsInputComponent_1;
let MultiItemsInputComponent = MultiItemsInputComponent_1 = class MultiItemsInputComponent {
    constructor() {
        this.disabled = false;
        this.values = new FormArray([]);
        this.onChange = (v) => { };
        this.onTouched = () => { };
    }
    ngOnInit() {
        this.subscription = this.values.valueChanges.subscribe(changedValues => {
            this.onChange(changedValues);
            this.onTouched();
        });
        const controls = {};
        controls[this.name] = this.values;
        this.formGroup = new FormGroup(controls);
    }
    get isRequired() {
        return typeof this.required !== 'undefined' || this.required === true;
    }
    get controls() {
        return this.values.controls;
    }
    ngOnDestroy() {
        if (this.subscription) {
            this.subscription.unsubscribe();
        }
    }
    registerOnChange(fn) {
        this.onChange = fn;
    }
    registerOnTouched(fn) {
        this.onTouched = fn;
    }
    setDisabledState(isDisabled) {
        this.disabled = isDisabled;
        if (this.disabled) {
            this.values.disable();
        }
        else {
            this.values.enable();
        }
    }
    writeValue(obj) {
        if (obj) {
            this.values.clear();
            obj.forEach(value => this.values.push(new FormControl({ value, disabled: this.disabled })));
        }
        else {
            this.values.clear();
        }
    }
    addControl() {
        this.values.push(new FormControl(''));
        this.onTouched();
    }
    removeValue(index) {
        this.values.removeAt(index);
        this.onTouched();
    }
};
__decorate([
    Input(),
    __metadata("design:type", String)
], MultiItemsInputComponent.prototype, "name", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], MultiItemsInputComponent.prototype, "required", void 0);
MultiItemsInputComponent = MultiItemsInputComponent_1 = __decorate([
    Component({
        selector: 'oav-multi-items-input',
        template: "<ng-container [formGroup]=\"formGroup\">\n  <div [formArrayName]=\"name\">\n    <div class=\"multi-item-entry\" *ngFor=\"let control of controls; let i = index\">\n      <input type=\"text\" [formControlName]=\"i.toString()\" (blur)=\"onTouched()\" />\n      <button type=\"button\" (click)=\"removeValue(i)\" [disabled]=\"disabled\">-</button>\n    </div>\n  </div>\n  <div class=\"oav-btn-container\">\n    <button type=\"button\" (click)=\"addControl()\" class=\"oav-add-btn\" [disabled]=\"disabled\">+</button>\n  </div>\n</ng-container>\n",
        providers: [
            {
                provide: NG_VALUE_ACCESSOR,
                useExisting: forwardRef(() => MultiItemsInputComponent_1),
                multi: true
            }
        ]
    }),
    __metadata("design:paramtypes", [])
], MultiItemsInputComponent);

let RequestsListComponent = class RequestsListComponent {
    constructor(openApiService) {
        this.openApiService = openApiService;
        this.openRequests = new Set();
        this.requests = [];
    }
    ngOnInit() { }
    set openRequest(req) {
        if (req) {
            this.openRequests.clear();
            this.openRequests.add(req);
        }
    }
    toggleRequest(req, state) {
        if (this.openRequests.has(req)) {
            this.openRequests.delete(req);
        }
        else {
            this.openRequests.add(req);
        }
    }
    getShortUrl(req) {
        if (req.request.url.match(/^.+?\/\/.+?(\/[^?]*)/)) {
            return RegExp.$1;
        }
        return req.request.url;
    }
    removeRequest(req) {
        this.requests.splice(this.requests.indexOf(req), 1);
        this.openRequests.delete(req);
        this.openApiService.removeRequest(req);
    }
};
RequestsListComponent.ctorParameters = () => [
    { type: OpenapiViewerService }
];
__decorate([
    Input(),
    __metadata("design:type", Array)
], RequestsListComponent.prototype, "requests", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [Object])
], RequestsListComponent.prototype, "openRequest", null);
RequestsListComponent = __decorate([
    Component({
        selector: 'oav-requests-list',
        template: "<div *ngFor=\"let req of requests\" [className]=\"'oav-request method-' + req.request.method.toLowerCase()\" [id]=\"req.id\">\n  <oav-method-header\n    [method]=\"req.request.method\"\n    [path]=\"getShortUrl(req)\"\n    [open]=\"openRequests.has(req)\"\n    (openChange)=\"toggleRequest(req, $event)\"\n  >\n    {{ req.startTs | date: 'medium' }}\n    <strong class=\"ml-2\">\n      <span [ngClass]=\"'status status-' + req.status\" *ngIf=\"!req.running && req.status\">{{ req.status }} {{ req.statusText }}</span>\n      <span class=\"loading\" *ngIf=\"req.running\"></span>\n    </strong>\n  </oav-method-header>\n\n  <oav-request-view [request]=\"req\" [hidden]=\"!openRequests.has(req)\" (dismiss)=\"removeRequest(req)\"></oav-request-view>\n</div>\n"
    }),
    __metadata("design:paramtypes", [OpenapiViewerService])
], RequestsListComponent);

let RequestViewComponent = class RequestViewComponent {
    constructor() {
        this.dismiss = new EventEmitter();
        this.displayLimitBytes = 1024 * 1024;
        this.displayMode = 'text';
        this.destroy = new Subject();
    }
    ngOnInit() {
        this.loadResponse();
    }
    ngOnDestroy() {
        this.destroy.complete();
    }
    loadResponse() {
        this.error = null;
        this.headers = '';
        this.body = '';
        this.request.httpEvents.pipe(takeUntil(this.destroy)).subscribe(status => {
            if (status.type === HttpEventType.ResponseHeader || status.type === HttpEventType.Response) {
                this.setHeaders(status.headers);
            }
            if (status.type === HttpEventType.Response) {
                this.setBody(status);
            }
        }, err => {
            console.error('error response', err);
            if (err instanceof HttpErrorResponse) {
                this.setErrorBody(err);
            }
            this.error = err.message;
        });
    }
    setErrorBody(err) {
        return __awaiter(this, void 0, void 0, function* () {
            if (err.error instanceof Blob) {
                this.updateDisplayMode(err.error.type);
                this.body = yield getTextFromBlob(err.error);
            }
        });
    }
    updateDisplayMode(mimeType) {
        if (mimeType.startsWith('text')) {
            this.displayMode = 'text';
        }
        else if (mimeType.includes('json')) {
            this.displayMode = 'json';
        }
        else if (mimeType.includes('xml')) {
            this.displayMode = 'xml';
        }
        else {
            this.displayMode = 'download';
        }
    }
    setBody(response) {
        return __awaiter(this, void 0, void 0, function* () {
            this.blob = response.body;
            const contentType = response.headers.get('content-type');
            const textTypes = /^(application\/.{0,30}(json|xml)|text\/)/;
            if (contentType.match(textTypes) && this.displayLimitBytes > this.blob.size) {
                this.displayMode = 'text';
                const body = yield getTextFromBlob(response.body);
                if (contentType.startsWith('application/json') && body && body.length) {
                    this.displayMode = 'json';
                    try {
                        const parsed = JSON.parse(body);
                        this.body = JSON.stringify(parsed, null, 2);
                    }
                    catch (e) {
                        this.body = body;
                        this.error = e;
                    }
                }
                else {
                    if (contentType.startsWith('application/xml')) {
                        this.displayMode = 'xml';
                    }
                    this.body = body;
                }
            }
            else {
                this.displayMode = 'download';
            }
        });
    }
    setHeaders(headers) {
        this.httpHeaders = headers;
        const headerLines = [];
        for (const headerName of headers.keys()) {
            for (const value of headers.getAll(headerName)) {
                headerLines.push(headerName + ': ' + value);
            }
        }
        this.headers = headerLines.join('\n');
    }
    getDuration() {
        if (this.request.endTs) {
            return this.request.endTs.getTime() - this.request.startTs.getTime();
        }
        else if (this.request.running) {
            return new Date().getTime() - this.request.startTs.getTime();
        }
        else {
            return null;
        }
    }
    doDismiss() {
        if (this.request.running) {
            this.request.cancel();
        }
        this.dismiss.next();
    }
    download() {
        const contentType = this.httpHeaders.get('content-type');
        const contentDisposition = this.httpHeaders.get('content-disposition');
        const contentType2Ending = {
            'application/json': 'json',
            'application/xml': 'xml',
            'text/plain': 'txt',
            'image/png': 'png',
            'image/gif': 'gif',
            'image/jpeg': 'jpg'
        };
        const readableTs = new Date().toISOString().replace(/[-:]|\.\d{3}/g, '');
        let name = readableTs + '_' + this.request.operationsItem.operation.operationId;
        const mime = Object.keys(contentType2Ending).find(mimeType => contentType.startsWith(mimeType));
        if (mime) {
            name += '.' + contentType2Ending[mime];
        }
        if (contentDisposition && contentDisposition.match(/filename="?(\w+)"?/)) {
            name = RegExp.$1;
        }
        const link = document.createElement('a');
        const url = URL.createObjectURL(this.blob);
        link.href = url;
        link.download = name;
        link.click();
        URL.revokeObjectURL(url);
    }
};
__decorate([
    Input(),
    __metadata("design:type", Object)
], RequestViewComponent.prototype, "request", void 0);
__decorate([
    Output(),
    __metadata("design:type", Object)
], RequestViewComponent.prototype, "dismiss", void 0);
RequestViewComponent = __decorate([
    Component({
        selector: 'oav-request-view',
        template: "<div class=\"oav-request-details\">\n  <div class=\"content\">\n    <div *ngIf=\"request.request.url\">\n      <div>\n        URL: <code>{{ request.request.url }}</code>\n      </div>\n    </div>\n    <div>\n      <div>\n        Status: <code>{{ request.status }} {{ request.statusText }}</code>\n      </div>\n    </div>\n    <div *ngIf=\"request.sentBytes\">\n      <div>\n        Bytes sent: <code>{{ request.sentBytes | number }}</code>\n      </div>\n    </div>\n\n    <h4>Response headers</h4>\n    <pre>{{ headers }}</pre>\n\n    <h4>\n      <span>Response body</span>\n      <span class=\"normal-text ml-2\">{{ request.receivedBytes | number }} bytes received after {{ getDuration() | number }} ms</span>\n    </h4>\n\n    <ng-container *ngIf=\"(displayMode === 'text' || displayMode === 'json' || displayMode === 'xml') && !request.running\">\n      <oav-code-input [ngModel]=\"body\" [mode]=\"displayMode\" [minLines]=\"5\" readonly></oav-code-input>\n    </ng-container>\n\n    <p class=\"oav-warning\" *ngIf=\"request.receivedBytes >= displayLimitBytes\">\n      Response is too large to be displayed. Please download it.\n    </p>\n\n    <pre *ngIf=\"error\" class=\"oav-error\">{{ error }}</pre>\n\n    <div class=\"d-flex justify-content-between\">\n      <button type=\"button\" class=\"download-btn\" (click)=\"download()\" *ngIf=\"!request.running\">Download</button>\n      <button type=\"button\" class=\"dismiss-btn\" (click)=\"doDismiss()\">{{ request.running ? 'Cancel' : 'Dismiss' }}</button>\n    </div>\n  </div>\n</div>\n"
    }),
    __metadata("design:paramtypes", [])
], RequestViewComponent);
function getTextFromBlob(blob) {
    return new Promise((resolve, reject) => {
        const reader = new FileReader();
        reader.onload = (ev) => {
            resolve(ev.target.result);
        };
        reader.onerror = err => {
            reject(err);
        };
        reader.readAsText(blob);
    });
}

let OpenableDirective = class OpenableDirective {
    constructor(element, renderer) {
        this.element = element;
        this.renderer = renderer;
        this.oavOpenable = true;
        this.open = false;
    }
    ngAfterViewInit() {
        if (this.oavOpenable !== false) {
            this.renderer.addClass(this.element.nativeElement, 'openable');
        }
    }
    onClick() {
        if (this.oavOpenable === false) {
            return;
        }
        this.open = !this.open;
        if (this.open) {
            this.renderer.addClass(this.element.nativeElement, 'open');
        }
        else {
            this.renderer.removeClass(this.element.nativeElement, 'open');
        }
    }
};
OpenableDirective.ctorParameters = () => [
    { type: ElementRef },
    { type: Renderer2 }
];
__decorate([
    Input(),
    __metadata("design:type", Object)
], OpenableDirective.prototype, "oavOpenable", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], OpenableDirective.prototype, "open", void 0);
__decorate([
    HostListener('click'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], OpenableDirective.prototype, "onClick", null);
OpenableDirective = __decorate([
    Directive({
        selector: '[oavOpenable]',
        exportAs: 'openable'
    }),
    __metadata("design:paramtypes", [ElementRef, Renderer2])
], OpenableDirective);

let MethodHeaderComponent = class MethodHeaderComponent {
    constructor() {
        this.open = false;
        this.openChange = new EventEmitter();
    }
    ngOnInit() { }
    toggleOpen() {
        this.open = !this.open;
        this.openChange.next(this.open);
    }
};
__decorate([
    Input(),
    __metadata("design:type", Object)
], MethodHeaderComponent.prototype, "open", void 0);
__decorate([
    Output(),
    __metadata("design:type", Object)
], MethodHeaderComponent.prototype, "openChange", void 0);
__decorate([
    Input(),
    __metadata("design:type", String)
], MethodHeaderComponent.prototype, "fragment", void 0);
__decorate([
    Input(),
    __metadata("design:type", String)
], MethodHeaderComponent.prototype, "method", void 0);
__decorate([
    Input(),
    __metadata("design:type", String)
], MethodHeaderComponent.prototype, "path", void 0);
MethodHeaderComponent = __decorate([
    Component({
        selector: 'oav-method-header',
        template: "<h3 (click)=\"toggleOpen()\" [className]=\"'oav-method-header oav-method-bg method-' + method.toLowerCase()\">\n  <a href=\"javascript:\" class=\"mainlink\" (click)=\"toggleOpen(); $event.stopPropagation()\">\n    <oav-method [method]=\"method\"></oav-method>\n    <span class=\"path ml-2\">{{ path }}</span>\n  </a>\n  <span class=\"summary\">\n    <ng-content></ng-content>\n  </span>\n  <button type=\"button\" class=\"toggle-button\" [class.open]=\"open\"></button>\n</h3>\n"
    }),
    __metadata("design:paramtypes", [])
], MethodHeaderComponent);

let MethodComponent = class MethodComponent {
};
__decorate([
    Input(),
    __metadata("design:type", String)
], MethodComponent.prototype, "method", void 0);
MethodComponent = __decorate([
    Component({
        selector: 'oav-method',
        template: "<span [ngClass]=\"'oav-method oav-method-' + method.toLowerCase()\">{{ method.toUpperCase() }}</span>\n",
        changeDetection: ChangeDetectionStrategy.OnPush
    })
], MethodComponent);

let IndexNavComponent = class IndexNavComponent {
    constructor(router, oavSettings, cd) {
        this.router = router;
        this.oavSettings = oavSettings;
        this.cd = cd;
        this.openTags = new Set();
        if (!this.oavSettings) {
            this.oavSettings = defaultOavSettings;
        }
        this.showHoverLabel = !!this.oavSettings.indexHoverLabel;
        this.showAuthentication = this.oavSettings.enableAuthentication;
        this.showModelsOverview = this.oavSettings.enableModelsOverview;
    }
    ngOnInit() {
        this.checkOpenTags(this.router.routerState.snapshot.url);
        this.sub = this.router.events.pipe(filter(ev => ev instanceof NavigationEnd)).subscribe((event) => {
            this.checkOpenTags(event.url);
            this.cd.markForCheck();
        });
        if (this.index && this.index.length === 1) {
            this.openTags.add(this.index[0].tag.name);
        }
    }
    ngOnDestroy() {
        this.sub.unsubscribe();
    }
    checkOpenTags(url) {
        if (url.match(/\/(.+)\/(.+)/)) {
            const tag = RegExp.$1;
            this.openTags.add(tag);
        }
    }
};
IndexNavComponent.ctorParameters = () => [
    { type: Router },
    { type: OavSettings, decorators: [{ type: Optional }] },
    { type: ChangeDetectorRef }
];
__decorate([
    Input(),
    __metadata("design:type", Array)
], IndexNavComponent.prototype, "index", void 0);
IndexNavComponent = __decorate([
    Component({
        selector: 'oav-index-nav',
        template: "<nav class=\"oav-index-nav\">\n  <ol class=\"oav-top-nav\">\n    <li>\n      <a routerLink=\"\" routerLinkActive=\"active\" [routerLinkActiveOptions]=\"{ exact: true }\">Overview</a>\n    </li>\n    <li *ngIf=\"showAuthentication\">\n      <a routerLink=\"/auth\" routerLinkActive=\"active\" [routerLinkActiveOptions]=\"{ exact: true }\">Authentication</a>\n    </li>\n    <li *ngIf=\"showModelsOverview\">\n      <a routerLink=\"/models\" routerLinkActive=\"active\" [routerLinkActiveOptions]=\"{ exact: true }\">Models overview</a>\n    </li>\n  </ol>\n\n  <ng-container *ngFor=\"let tag of index\">\n    <header oavOpenable #openable=\"openable\" [open]=\"openTags.has(tag.tag.name)\">\n      <a href=\"javascript:\">\n        <span>{{ tag.tag.name }}</span>\n        <button type=\"button\" class=\"toggle-button ml-auto\" [class.open]=\"openable.open\"></button>\n      </a>\n    </header>\n    <ol [hidden]=\"!openable.open\">\n      <oav-index-nav-endpoints [tag]=\"tag\"></oav-index-nav-endpoints>\n    </ol>\n  </ng-container>\n\n  <ol class=\"oav-bottom-nav\">\n    <li>\n      <span class=\"ml-3\">Powered by</span><br />\n      <a href=\"https://github.com/dhcode/openapi-ui\" target=\"_blank\">OpenAPI UI</a>\n    </li>\n  </ol>\n</nav>\n",
        changeDetection: ChangeDetectionStrategy.OnPush
    }),
    __param(1, Optional()),
    __metadata("design:paramtypes", [Router, OavSettings, ChangeDetectorRef])
], IndexNavComponent);

let OperationViewComponent = class OperationViewComponent {
    constructor(route, openApiService) {
        this.route = route;
        this.openApiService = openApiService;
        this.pathItem = null;
        this.operationItem = null;
        this.tag = null;
        this.error = null;
    }
    ngOnInit() {
        this.sub = combineLatest(this.route.params, this.openApiService.tagIndex).subscribe(([params, index]) => {
            this.loadData(index, params.tag, params.opId);
        });
    }
    ngOnDestroy() {
        if (this.sub) {
            this.sub.unsubscribe();
        }
    }
    loadData(index, tag, opId) {
        this.pathItem = null;
        this.operationItem = null;
        this.tag = tag;
        const tagIndex = index.find(idx => idx.tag.name === tag);
        if (!tagIndex) {
            this.error = 'noTagFound';
            return;
        }
        for (const pi of tagIndex.paths) {
            const operationItem = pi.operations.find(op => op.operation.operationId === opId);
            if (operationItem) {
                this.pathItem = pi;
                this.operationItem = operationItem;
            }
        }
        if (!this.operationItem) {
            this.error = 'noOperationFound';
        }
    }
};
OperationViewComponent.ctorParameters = () => [
    { type: ActivatedRoute },
    { type: OpenapiViewerService }
];
OperationViewComponent = __decorate([
    Component({
        selector: 'oav-operation-view',
        template: "<div class=\"oav-content\">\n  <ng-container *ngIf=\"error === 'noOperationFound'\">\n    Operation was not found\n  </ng-container>\n  <ng-container *ngIf=\"error === 'noTagFound'\">\n    Tag was not found\n  </ng-container>\n  <ng-container *ngIf=\"operationItem\">\n    <oav-operation [tag]=\"tag\" [operationItem]=\"operationItem\" [pathItem]=\"pathItem\"></oav-operation>\n  </ng-container>\n</div>\n"
    }),
    __metadata("design:paramtypes", [ActivatedRoute, OpenapiViewerService])
], OperationViewComponent);

let OverviewViewComponent = class OverviewViewComponent {
    constructor(openApiService, authService, cd) {
        this.openApiService = openApiService;
        this.authService = authService;
        this.cd = cd;
        this.index = [];
        this.spec = null;
    }
    ngOnInit() {
        this.sub = combineLatest([this.openApiService.spec, this.openApiService.tagIndex]).subscribe(([spec, index]) => {
            this.spec = spec;
            this.index = index;
            this.cd.detectChanges();
        });
    }
    ngOnDestroy() {
        this.sub.unsubscribe();
    }
};
OverviewViewComponent.ctorParameters = () => [
    { type: OpenapiViewerService },
    { type: OpenapiAuthService },
    { type: ChangeDetectorRef }
];
OverviewViewComponent = __decorate([
    Component({
        selector: 'oav-overview-view',
        template: "<div class=\"oav-content\">\n  <h1>{{ spec.info.title }}</h1>\n\n  <p *ngIf=\"spec.info.description\">\n    <markdown [data]=\"spec.info.description\"></markdown>\n  </p>\n\n  <div *ngIf=\"spec.info.termsOfService\">\n    <a [href]=\"spec.info.termsOfService\" target=\"_blank\">Terms of service</a>\n  </div>\n  <div *ngIf=\"spec.info.contact?.url\">\n    <a [href]=\"spec.info.contact.url\" target=\"_blank\">{{ spec.info.contact.name || 'Contact link' }}</a>\n  </div>\n  <div *ngIf=\"spec.info.contact?.email\">\n    <a [href]=\"'mailto:' + spec.info.contact.email\" target=\"_blank\">{{ spec.info.contact.name || 'Contact email' }}</a>\n  </div>\n  <div *ngIf=\"spec.info.license?.url\">\n    <a [href]=\"spec.info.license?.url\" target=\"_blank\">License: {{ spec.info.license.name }}</a>\n  </div>\n  <div *ngIf=\"spec.info.license && !spec.info.license.url\">License: {{ spec.info.license.name }}</div>\n\n  <ng-container *ngFor=\"let tag of index\">\n    <h2>{{ tag.tag.name }}</h2>\n    <p *ngIf=\"tag.tag.description\">{{ tag.tag.description }}</p>\n    <small *ngIf=\"tag.tag.externalDocs\" class=\"extra-link\">\n      {{ tag.tag.externalDocs.description }}:\n      <a [href]=\"tag.tag.externalDocs.url\" target=\"_blank\">{{ tag.tag.externalDocs.url }}</a>\n    </small>\n\n    <oav-overview-view-endpoints [tag]=\"tag\"></oav-overview-view-endpoints>\n  </ng-container>\n</div>\n",
        changeDetection: ChangeDetectionStrategy.OnPush
    }),
    __metadata("design:paramtypes", [OpenapiViewerService, OpenapiAuthService, ChangeDetectorRef])
], OverviewViewComponent);

let AuthenticationViewComponent = class AuthenticationViewComponent {
    constructor(authService, route) {
        this.authService = authService;
        this.route = route;
        this.schemes = [];
        this.sub = null;
    }
    ngOnInit() {
        this.sub = this.authService.securitySchemes.subscribe(schemas => (this.schemes = schemas));
        this.route.fragment.subscribe(fragment => {
            if (fragment && fragment.length) {
                const params = parseQueryString(fragment);
                this.authService.handleOAuthCallback(params);
            }
        });
        this.route.queryParams.subscribe(params => {
            if (params.state) {
                this.authService.handleOAuthCallback(params);
            }
        });
    }
    ngOnDestroy() {
        if (this.sub) {
            this.sub.unsubscribe();
        }
    }
};
AuthenticationViewComponent.ctorParameters = () => [
    { type: OpenapiAuthService },
    { type: ActivatedRoute }
];
AuthenticationViewComponent = __decorate([
    Component({
        selector: 'oav-authentication-view',
        template: "<div class=\"oav-content\">\n  <h1>Authentication schemes</h1>\n\n  <div *ngFor=\"let scheme of schemes\">\n    <h3>\n      {{ scheme.name }}\n      <span class=\"oav-auth-status ml-1\" [class.required]=\"!scheme.authenticated\" [class.ok]=\"scheme.authenticated\"></span>\n      <span class=\"ml-1 normal-text\">({{ scheme.securityScheme.type }})</span>\n    </h3>\n\n    <markdown *ngIf=\"scheme.securityScheme.description\" [data]=\"scheme.securityScheme.description\"></markdown>\n\n    <oav-auth-credentials [securityScheme]=\"scheme\"></oav-auth-credentials>\n  </div>\n\n  <p *ngIf=\"!schemes.length\">\n    No authentication schemes available.\n  </p>\n</div>\n"
    }),
    __metadata("design:paramtypes", [OpenapiAuthService, ActivatedRoute])
], AuthenticationViewComponent);

const categoryLabels = {
    schema: 'Schemas',
    response: 'Responses',
    parameter: 'Parameters',
    example: 'Examples',
    requestBody: 'RequestBodies',
    header: 'Headers',
    securityScheme: 'SecuritySchemes',
    link: 'Links',
    callback: 'Callbacks'
};
let ModelsViewComponent = class ModelsViewComponent {
    constructor(openApiService, oavSettings, cd) {
        this.openApiService = openApiService;
        this.oavSettings = oavSettings;
        this.cd = cd;
        this.categories = [];
        if (!this.oavSettings) {
            this.oavSettings = defaultOavSettings;
        }
    }
    get showRaw() {
        return this.oavSettings.showRawModelDefinition;
    }
    ngOnInit() {
        this.sub = this.openApiService.spec.subscribe(spec => {
            this.categories = identifyModels(spec);
            this.cd.detectChanges();
        });
    }
    ngOnDestroy() {
        this.sub.unsubscribe();
    }
};
ModelsViewComponent.ctorParameters = () => [
    { type: OpenapiViewerService },
    { type: OavSettings, decorators: [{ type: Optional }] },
    { type: ChangeDetectorRef }
];
ModelsViewComponent = __decorate([
    Component({
        selector: 'oav-models-view',
        template: "<div class=\"oav-content\">\n  <h1>Definitions</h1>\n\n  <ng-container *ngFor=\"let cat of categories\">\n    <h2>{{ cat.label }}</h2>\n\n    <div *ngFor=\"let def of cat.definitions\">\n      <h3>{{ def.name }}</h3>\n      <div class=\"ml-3\" *ngIf=\"def.schema\">\n        <oav-json-schema [schema]=\"def.schema\" [defaultModelName]=\"def.name\"></oav-json-schema>\n\n        <ng-container *ngIf=\"showRaw\">\n          <h4 oavOpenable #rawOpenable=\"openable\" class=\"toggle-button\">Raw {{ def.name }} model definition</h4>\n          <oav-code-input *ngIf=\"rawOpenable.open\" mode=\"json\" [ngModel]=\"def.schema | json\" readonly></oav-code-input>\n        </ng-container>\n      </div>\n    </div>\n  </ng-container>\n</div>\n",
        changeDetection: ChangeDetectionStrategy.OnPush
    }),
    __param(1, Optional()),
    __metadata("design:paramtypes", [OpenapiViewerService, OavSettings, ChangeDetectorRef])
], ModelsViewComponent);
function getCategory(propName, categories) {
    const existing = categories.find(c => c.name === propName);
    if (existing) {
        return existing;
    }
    const cat = {
        name: propName,
        label: categoryLabels[propName],
        definitions: []
    };
    categories.push(cat);
    return cat;
}
function addToCategory(defItem, categories) {
    const categoryPropsNames = Object.keys(categoryLabels);
    for (const propName of categoryPropsNames) {
        if (defItem[propName]) {
            const cat = getCategory(propName, categories);
            cat.definitions.push(defItem);
        }
    }
}
function identifyModels(spec) {
    const categories = [];
    // OASv2
    if (spec.definitions) {
        Object.keys(spec.definitions)
            .map(name => getDefinitionsItem('schema', name, spec.definitions[name]))
            .forEach(defItem => addToCategory(defItem, categories));
    }
    if (spec.responses) {
        Object.keys(spec.responses)
            .map(name => getDefinitionsItem('response', name, spec.definitions[name]))
            .forEach(defItem => addToCategory(defItem, categories));
    }
    if (spec.parameters) {
        Object.keys(spec.parameters)
            .map(name => getDefinitionsItem('parameter', name, spec.definitions[name]))
            .forEach(defItem => addToCategory(defItem, categories));
    }
    // OASv3
    if (spec.components) {
        Object.keys(spec.components).forEach(categoryPlural => {
            const category = categoryPlural.substr(0, categoryPlural.length - 1);
            const cat = getCategory(category, categories);
            Object.keys(spec.components[categoryPlural])
                .map(name => getDefinitionsItem(category, name, spec.components[categoryPlural][name]))
                .forEach(defItem => {
                cat.definitions.push(defItem);
            });
        });
    }
    return categories;
}
function getDefinitionsItem(category, name, component) {
    const item = { name };
    item[category] = component;
    return item;
}

const routes = [
    {
        path: '',
        pathMatch: 'full',
        component: OverviewViewComponent
    },
    {
        path: 'auth',
        component: AuthenticationViewComponent
    },
    {
        path: 'models',
        component: ModelsViewComponent
    },
    {
        path: ':tag/:opId',
        component: OperationViewComponent
    }
];
let OpenapiViewerRoutingModule = class OpenapiViewerRoutingModule {
};
OpenapiViewerRoutingModule = __decorate([
    NgModule({
        imports: [RouterModule.forRoot(routes, { useHash: true })],
        exports: [RouterModule]
    })
], OpenapiViewerRoutingModule);

let TabNavItemDirective = class TabNavItemDirective {
    constructor(templateRef) {
        this.templateRef = templateRef;
        this.id = randomHex(8);
        this.active = false;
    }
    set oavTabNavItem(label) {
        this.label = label;
    }
    set oavTabNavItemId(id) {
        this.id = id;
    }
    set oavTabNavItemDetails(details) {
        this.details = details;
    }
};
TabNavItemDirective.ctorParameters = () => [
    { type: TemplateRef }
];
__decorate([
    Input(),
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [Object])
], TabNavItemDirective.prototype, "oavTabNavItem", null);
__decorate([
    Input(),
    __metadata("design:type", String),
    __metadata("design:paramtypes", [String])
], TabNavItemDirective.prototype, "oavTabNavItemId", null);
__decorate([
    Input(),
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [Object])
], TabNavItemDirective.prototype, "oavTabNavItemDetails", null);
TabNavItemDirective = __decorate([
    Directive({
        selector: '[oavTabNavItem]'
    }),
    __metadata("design:paramtypes", [TemplateRef])
], TabNavItemDirective);

let TabNavComponent = class TabNavComponent {
    constructor() {
        this.tabs = [];
        this.tabChange = new EventEmitter();
        this.destroy = new Subject();
    }
    set tabList(tabs) {
        if (tabs) {
            this.tabs = tabs.toArray();
            this.checkForActiveTab();
        }
    }
    ngOnChanges(changes) {
        if (changes.tab && !changes.tab.firstChange) {
            this.showTab(this.tab);
        }
    }
    ngOnDestroy() {
        this.destroy.complete();
    }
    checkForActiveTab() {
        if (!this.tabs.find(t => t.active) && this.tabs.length) {
            if (this.tab && this.getTabById(this.tab)) {
                this.showTab(this.tab);
            }
            else {
                this.showTab(this.tabs[0].id);
            }
        }
    }
    getTabById(id) {
        return this.tabs.find(tab => tab.id === id);
    }
    showTab(id, notify = false) {
        for (const tab of this.tabs) {
            if (tab.active && tab.id !== id) {
                tab.active = false;
            }
            if (!tab.active && tab.id === id) {
                tab.active = true;
                this.tab = tab.id;
                this.tabContent = tab.templateRef;
                if (notify) {
                    this.tabChange.next(tab.id);
                }
            }
        }
    }
};
__decorate([
    Output(),
    __metadata("design:type", Object)
], TabNavComponent.prototype, "tabChange", void 0);
__decorate([
    Input(),
    __metadata("design:type", String)
], TabNavComponent.prototype, "tab", void 0);
__decorate([
    ContentChildren(TabNavItemDirective, { descendants: true }),
    __metadata("design:type", QueryList),
    __metadata("design:paramtypes", [QueryList])
], TabNavComponent.prototype, "tabList", null);
TabNavComponent = __decorate([
    Component({
        selector: 'oav-tab-nav',
        template: "<nav class=\"oav-tab-nav\">\n  <a href=\"javascript:\" *ngFor=\"let tab of tabs\" [class.active]=\"tab.active\" (click)=\"showTab(tab.id, true)\">\n    <oav-template-outlet [tpl]=\"tab.label\" [ctx]=\"tab\"></oav-template-outlet>\n  </a>\n</nav>\n<div class=\"oav-tab-content\">\n  <ng-container *ngTemplateOutlet=\"tabContent\"></ng-container>\n</div>\n"
    }),
    __metadata("design:paramtypes", [])
], TabNavComponent);

let TemplateOutletComponent = class TemplateOutletComponent {
    isTemplate() {
        return this.tpl instanceof TemplateRef;
    }
};
__decorate([
    Input(),
    __metadata("design:type", Object)
], TemplateOutletComponent.prototype, "tpl", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], TemplateOutletComponent.prototype, "ctx", void 0);
TemplateOutletComponent = __decorate([
    Component({
        selector: 'oav-template-outlet',
        template: "<ng-container *ngIf=\"isTemplate(); else noTpl\">\n  <ng-container *ngTemplateOutlet=\"tpl; context: ctx\"></ng-container>\n</ng-container>\n<ng-template #noTpl>{{ tpl }}</ng-template>\n",
        changeDetection: ChangeDetectionStrategy.OnPush
    })
], TemplateOutletComponent);

let RequestBodyInputComponent = class RequestBodyInputComponent {
    constructor() {
        this.formGroup = new FormGroup({
            contentType: new FormControl()
        });
        this.contentParameters = {};
        this.contentTypes = [];
        this.currentParameters = [];
    }
    ngOnInit() {
        this.addToFormGroup.addControl('requestBody', this.formGroup);
    }
    ngOnDestroy() {
        this.addToFormGroup.removeControl('requestBody');
    }
    ngOnChanges(changes) {
        this.identifyParameters();
    }
    identifyParameters() {
        this.contentParameters = {};
        this.contentTypes = [];
        if (!this.requestBody || !this.requestBody.content) {
            return;
        }
        this.contentTypes = Object.keys(this.requestBody.content);
        if (this.contentTypes.length) {
            for (const type of this.contentTypes) {
                this.contentParameters[type] = this.contentToParameter(type, this.requestBody.content[type]);
            }
            this.currentContentType = this.contentTypes[0];
            this.currentParameters = this.contentParameters[this.currentContentType];
            this.formGroup.patchValue({ contentType: this.currentContentType });
        }
    }
    contentToParameter(mediaType, def) {
        if ((mediaType === 'application/x-www-form-urlencoded' || mediaType.startsWith('multipart')) && def.schema) {
            const schema = def.schema;
            if ((schema.type === 'object' || !schema.type) && schema.properties) {
                return Object.keys(schema.properties).map(key => {
                    const propScheme = schema.properties[key];
                    return {
                        name: key,
                        customIn: 'form-data',
                        required: schema.required && schema.required.includes(key),
                        description: propScheme.description,
                        schema: propScheme
                    };
                });
            }
        }
        const param = {
            name: 'requestBody',
            customIn: 'body',
            required: this.requestBody.required,
            description: this.requestBody.description,
            schema: def.schema,
            example: def.example,
            examples: def.examples
        };
        return [param];
    }
};
__decorate([
    Input(),
    __metadata("design:type", Object)
], RequestBodyInputComponent.prototype, "requestBody", void 0);
__decorate([
    Input(),
    __metadata("design:type", FormGroup)
], RequestBodyInputComponent.prototype, "addToFormGroup", void 0);
RequestBodyInputComponent = __decorate([
    Component({
        selector: 'oav-request-body-input',
        template: "<ng-container *ngFor=\"let param of currentParameters\">\n  <oav-parameter [parameter]=\"param\" [formGroup]=\"formGroup\"></oav-parameter>\n</ng-container>\n"
    }),
    __metadata("design:paramtypes", [])
], RequestBodyInputComponent);

let AuthCredentialsComponent = class AuthCredentialsComponent {
    constructor(authService) {
        this.authService = authService;
        this.displayMode = 'unknown';
        this.username = new FormControl('');
        this.password = new FormControl('');
        this.apiKey = new FormControl('');
        this.clientId = new FormControl('');
        this.clientSecret = new FormControl('');
        this.scopes = new FormArray([]);
        this.flow = new FormControl('');
        this.remember = new FormControl(false);
        this.scopesInfo = [];
        this.loading = false;
        this.error = null;
        /**
         * The available flows
         */
        this.flows = [];
    }
    ngOnInit() {
        this.sub = this.flow.valueChanges.subscribe(flow => {
            this.readOAuthCredentials(flow);
        });
    }
    ngOnChanges(changes) {
        if (changes.securityScheme && this.securityScheme) {
            this.checkScheme();
        }
    }
    ngOnDestroy() {
        this.sub.unsubscribe();
    }
    checkScheme() {
        const type = this.securityScheme.securityScheme.type;
        if (type === 'apiKey') {
            this.displayMode = 'apiKey';
            this.readApiKey();
        }
        else if (type === 'http' || type === 'basic') {
            this.displayMode = 'usernamePassword';
            this.readHttpCredentials();
        }
        else if (type === 'oauth2' || type === 'openIdConnect') {
            this.displayMode = 'oauth';
            this.readOAuthCredentials();
        }
        else {
            this.displayMode = 'unknown';
        }
        if (this.securityScheme.remember) {
            this.remember.patchValue(true);
        }
    }
    readApiKey() {
        this.formGroup = new FormGroup({ apiKey: this.apiKey, remember: this.remember });
        if (typeof this.securityScheme.credentials === 'string') {
            this.apiKey.patchValue(this.securityScheme.credentials);
        }
        else {
            this.apiKey.patchValue('');
        }
    }
    readHttpCredentials() {
        this.formGroup = new FormGroup({ username: this.username, password: this.password, remember: this.remember });
        const credentials = this.securityScheme.credentials;
        if (typeof credentials === 'object' && credentials) {
            if (typeof credentials.username === 'string') {
                this.username.patchValue(credentials.username);
            }
            else {
                this.username.patchValue('');
            }
            if (typeof credentials.password === 'string') {
                this.password.patchValue(credentials.password);
            }
            else {
                this.password.patchValue('');
            }
        }
        else {
            this.username.patchValue('');
            this.password.patchValue('');
        }
    }
    readOAuthCredentials(flow) {
        const credentials = this.securityScheme.credentials;
        if (!flow && credentials && credentials.flow) {
            flow = credentials.flow;
        }
        this.flows = identifyFlows(this.securityScheme.securityScheme);
        const currentFlow = flow ? this.flows.find(f => f.flow === flow) : this.flows[0];
        if (!currentFlow) {
            return;
        }
        if (credentials) {
            this.token = credentials.token;
        }
        this.currentFlow = currentFlow;
        this.scopesInfo = currentFlow.scopes;
        this.formGroup = new FormGroup({
            clientId: this.clientId,
            clientSecret: this.clientSecret,
            username: this.username,
            password: this.password,
            scopes: this.scopes,
            remember: this.remember
        });
        this.scopes.clear();
        this.remember.patchValue(true);
        this.flow.patchValue(currentFlow.flow);
        if (typeof credentials === 'object' && credentials) {
            this.clientId.patchValue(credentials.clientId);
            this.clientSecret.patchValue(credentials.clientSecret);
            this.username.patchValue(credentials.username || '');
            this.password.patchValue(credentials.password || '');
            for (const scope of this.scopesInfo) {
                this.scopes.push(new FormControl(credentials.scopes.includes(scope.scope)));
            }
        }
        else {
            this.clientId.patchValue('');
            for (const scope of this.scopesInfo) {
                this.scopes.push(new FormControl(true));
            }
        }
    }
    save() {
        if (this.displayMode === 'apiKey') {
            this.authService.updateCredentials(this.securityScheme.name, this.apiKey.value, this.apiKey.value.length > 0, this.remember.value);
        }
        if (this.displayMode === 'usernamePassword') {
            this.authService.updateCredentials(this.securityScheme.name, {
                username: this.username.value,
                password: this.password.value
            }, this.username.value.length > 0, this.remember.value);
        }
        if (this.displayMode === 'oauth') {
            const scopes = this.scopesInfo.map((s, i) => (this.scopes.value[i] ? s.scope : null)).filter(s => !!s);
            const credentials = {
                clientId: this.clientId.value,
                clientSecret: this.clientSecret.value,
                username: this.username.value,
                password: this.password.value,
                flow: this.flow.value,
                scopes,
                token: null,
                nonce: randomHex(16),
                redirectUri: getCurrentUrl()
            };
            this.loading = true;
            this.error = null;
            this.authService.runOAuthAuthorization(this.securityScheme.name, credentials).subscribe(() => {
                this.loading = false;
            }, err => {
                this.loading = false;
                this.error = err;
            });
        }
        this.formGroup.markAsPristine();
    }
    removeToken() {
        this.authService.removeToken(this.securityScheme.name);
    }
};
AuthCredentialsComponent.ctorParameters = () => [
    { type: OpenapiAuthService }
];
__decorate([
    Input(),
    __metadata("design:type", Object)
], AuthCredentialsComponent.prototype, "securityScheme", void 0);
AuthCredentialsComponent = __decorate([
    Component({
        selector: 'oav-auth-credentials',
        template: "<form action=\"javascript:\" (submit)=\"save()\" *ngIf=\"formGroup\" [formGroup]=\"formGroup\">\n  <ng-container *ngIf=\"displayMode === 'apiKey'\">\n    <div class=\"oav-parameter\">\n      <div class=\"name\">\n        <div><strong>API Key</strong>:</div>\n        <div class=\"param-in\">\n          {{ securityScheme.securityScheme.in }}\n          <span class=\"format\" *ngIf=\"securityScheme.securityScheme.scheme\">({{ securityScheme.securityScheme.scheme }})</span>\n        </div>\n      </div>\n      <div class=\"input\">\n        <input [formControl]=\"apiKey\" name=\"apiKey\" />\n      </div>\n    </div>\n    <button class=\"send-btn\" [disabled]=\"formGroup.pristine\">Save</button>\n    <label class=\"ml-2\">\n      <input type=\"checkbox\" [formControl]=\"remember\" />\n      Remember in browser session\n    </label>\n  </ng-container>\n  <ng-container *ngIf=\"displayMode === 'usernamePassword'\">\n    <div class=\"oav-parameter\">\n      <div class=\"name\">\n        <div><strong>Username</strong>:</div>\n        <div class=\"param-in\">\n          {{ securityScheme.securityScheme.in }}\n          <span class=\"format\" *ngIf=\"securityScheme.securityScheme.scheme\">({{ securityScheme.securityScheme.scheme }})</span>\n        </div>\n      </div>\n      <div class=\"input\">\n        <input [formControl]=\"username\" name=\"username\" />\n      </div>\n    </div>\n    <div class=\"oav-parameter\">\n      <div class=\"name\">\n        <div><strong>Password</strong>:</div>\n        <div class=\"param-in\">\n          {{ securityScheme.securityScheme.in }}\n          <span class=\"format\" *ngIf=\"securityScheme.securityScheme.scheme\">({{ securityScheme.securityScheme.scheme }})</span>\n        </div>\n      </div>\n      <div class=\"input\">\n        <input [formControl]=\"password\" type=\"password\" name=\"password\" />\n      </div>\n    </div>\n    <button class=\"send-btn\" [disabled]=\"formGroup.pristine\">Save</button>\n    <label class=\"ml-2\">\n      <input type=\"checkbox\" [formControl]=\"remember\" />\n      Remember in browser session\n    </label>\n  </ng-container>\n  <ng-container *ngIf=\"displayMode === 'oauth'\">\n    <div class=\"oav-parameter\" *ngIf=\"token\">\n      <div class=\"name\">\n        <div><strong>Current token</strong>:</div>\n      </div>\n      <div class=\"input\">\n        <div>\n          Access token: <code>{{ token.access_token }}</code>\n        </div>\n        <div>\n          Token type: <code>{{ token.token_type }}</code>\n        </div>\n        <div *ngIf=\"token.expires_at\">Expires {{ token.expires_at | date: 'long' }}</div>\n        <button type=\"button\" class=\"dismiss-btn\" (click)=\"removeToken()\">Remove token</button>\n      </div>\n    </div>\n    <div class=\"oav-parameter\">\n      <div class=\"name\">\n        <div><strong>Flow</strong>:</div>\n      </div>\n      <div class=\"input\">\n        <select [formControl]=\"flow\" name=\"flow\">\n          <option *ngFor=\"let f of flows\" [value]=\"f.flow\">{{ f.flow }}</option>\n        </select>\n      </div>\n    </div>\n    <div class=\"oav-parameter\">\n      <div class=\"name\">\n        <div><strong>Client ID</strong>:</div>\n      </div>\n      <div class=\"input\">\n        <input [formControl]=\"clientId\" name=\"clientId\" />\n      </div>\n    </div>\n    <div class=\"oav-parameter\" *ngIf=\"flow.value === 'authorizationCode' || flow.value === 'clientCredentials'\">\n      <div class=\"name\">\n        <div><strong>Client Secret</strong>:</div>\n      </div>\n      <div class=\"input\">\n        <input [formControl]=\"clientSecret\" type=\"password\" name=\"clientSecret\" />\n      </div>\n    </div>\n    <div class=\"oav-parameter\" *ngIf=\"flow.value === 'password'\">\n      <div class=\"name\">\n        <div><strong>Username</strong>:</div>\n      </div>\n      <div class=\"input\">\n        <input [formControl]=\"username\" name=\"username\" />\n      </div>\n    </div>\n    <div class=\"oav-parameter\" *ngIf=\"flow.value === 'password'\">\n      <div class=\"name\">\n        <div><strong>Password</strong>:</div>\n      </div>\n      <div class=\"input\">\n        <input [formControl]=\"password\" type=\"password\" name=\"password\" />\n      </div>\n    </div>\n    <div class=\"oav-parameter\">\n      <div class=\"name\">\n        <div><strong>Scopes</strong>:</div>\n        <div class=\"param-in\">\n          {{ securityScheme.securityScheme.in }}\n          <span class=\"format\" *ngIf=\"securityScheme.securityScheme.scheme\">({{ securityScheme.securityScheme.scheme }})</span>\n        </div>\n      </div>\n      <div class=\"input\" [formArrayName]=\"'scopes'\">\n        <div *ngFor=\"let scope of scopesInfo; let i = index\">\n          <label>\n            <input type=\"checkbox\" [formControlName]=\"i\" [value]=\"scope.scope\" />\n            {{ scope.description }}\n          </label>\n        </div>\n      </div>\n    </div>\n    <button class=\"send-btn\" [disabled]=\"formGroup.invalid || loading\" [class.loading]=\"loading\">Authenticate</button>\n    <label class=\"ml-2\">\n      <input type=\"checkbox\" [formControl]=\"remember\" required />\n      Remember in browser session\n    </label>\n    <div class=\"oav-error\" *ngIf=\"error\">{{ error }}</div>\n  </ng-container>\n</form>\n<ng-container *ngIf=\"displayMode === 'unknown'\">\n  <p>Unsupported authentication scheme '{{ securityScheme?.securityScheme?.type }}'</p>\n</ng-container>\n"
    }),
    __metadata("design:paramtypes", [OpenapiAuthService])
], AuthCredentialsComponent);
function getCurrentUrl() {
    const url = window.location.href;
    const idx = url.indexOf('?');
    if (idx !== -1) {
        return url.slice(0, idx);
    }
    return url;
}

let AuthStatusComponent = class AuthStatusComponent {
    constructor(oavSettings, authService, cd) {
        this.oavSettings = oavSettings;
        this.authService = authService;
        this.cd = cd;
        this.schemeNames = [];
        if (!this.oavSettings) {
            this.oavSettings = defaultOavSettings;
        }
        this.showAuthentication = this.oavSettings.enableAuthentication;
    }
    ngOnInit() {
        this.sub = this.authService.updatedCredentials.subscribe(schemeName => {
            if (this.schemeNames.includes(schemeName)) {
                this.cd.detectChanges();
            }
        });
    }
    ngOnDestroy() {
        this.sub.unsubscribe();
    }
    ngOnChanges(changes) {
        if (changes.operation) {
            this.authStatus = this.authService.getAuthStatus(this.operation.security);
            this.requiredSchemes = this.authService.getRequiredSchemes(this.operation.security);
            this.schemeNames = [];
            for (const reqs of this.requiredSchemes) {
                for (const scheme of reqs) {
                    this.schemeNames.push(scheme.name);
                }
            }
        }
    }
};
AuthStatusComponent.ctorParameters = () => [
    { type: OavSettings, decorators: [{ type: Optional }] },
    { type: OpenapiAuthService },
    { type: ChangeDetectorRef }
];
__decorate([
    Input(),
    __metadata("design:type", Object)
], AuthStatusComponent.prototype, "operation", void 0);
__decorate([
    ViewChild(CdkPortal, { static: true }),
    __metadata("design:type", CdkPortal)
], AuthStatusComponent.prototype, "portal", void 0);
__decorate([
    ViewChild('statusIcon'),
    __metadata("design:type", ElementRef)
], AuthStatusComponent.prototype, "statusIcon", void 0);
AuthStatusComponent = __decorate([
    Component({
        selector: 'oav-auth-status',
        template: "<span\n  #statusIcon\n  class=\"oav-auth-status ml-1\"\n  *ngIf=\"showAuthentication\"\n  [class.required]=\"authStatus === 'required'\"\n  [class.ok]=\"authStatus === 'ok'\"\n  [oavTooltip]=\"authStatus !== 'none' ? portal : null\"\n></span>\n\n<oav-tooltip *cdkPortal>\n  <ng-container *ngFor=\"let reqs of requiredSchemes\">\n    <div *ngFor=\"let status of reqs\">\n      <h4 class=\"m-0\">\n        {{ status.name }}\n        <span class=\"oav-auth-status ml-1\" [class.required]=\"!status.authenticated\" [class.ok]=\"status.authenticated\"></span>\n        <span class=\"ml-1 normal-text\" *ngIf=\"status.authenticated\">authenticated</span>\n        <span class=\"ml-1 normal-text\" *ngIf=\"!status.authenticated\">not provided</span>\n      </h4>\n      <ng-container *ngIf=\"status.scopes?.length\"> Required scopes: {{ status.scopes.join(', ') }} </ng-container>\n    </div>\n  </ng-container>\n</oav-tooltip>\n",
        changeDetection: ChangeDetectionStrategy.OnPush
    }),
    __param(0, Optional()),
    __metadata("design:paramtypes", [OavSettings, OpenapiAuthService, ChangeDetectorRef])
], AuthStatusComponent);

let TooltipComponent = class TooltipComponent {
    constructor() { }
    ngOnInit() { }
};
TooltipComponent = __decorate([
    Component({
        selector: 'oav-tooltip',
        template: "<ng-content></ng-content>\n"
    }),
    __metadata("design:paramtypes", [])
], TooltipComponent);

let TooltipDirective = class TooltipDirective {
    constructor(overlay, elementRef, viewContainer) {
        this.overlay = overlay;
        this.elementRef = elementRef;
        this.viewContainer = viewContainer;
        this.fixed = false;
    }
    onClick(e) {
        e.stopPropagation();
        this.fixed = true;
        this.open();
    }
    onMouseEnter(e) {
        this.open();
    }
    onMouseLeave(e) {
        if (!this.fixed) {
            this.close();
        }
    }
    onWindowClick() {
        this.close();
    }
    ngOnDestroy() {
        this.close();
    }
    open() {
        if (this.overlayRef || !this.oavTooltip) {
            return;
        }
        this.overlayRef = this.overlay.create({
            positionStrategy: this.overlay
                .position()
                .flexibleConnectedTo(this.elementRef)
                .withPositions([
                {
                    overlayX: 'start',
                    overlayY: 'top',
                    originX: 'start',
                    originY: 'bottom'
                }
            ])
        });
        this.overlayRef.attach(this.oavTooltip);
    }
    close() {
        this.fixed = false;
        if (this.overlayRef) {
            this.overlayRef.dispose();
            this.overlayRef = null;
        }
    }
};
TooltipDirective.ctorParameters = () => [
    { type: Overlay },
    { type: ElementRef },
    { type: ViewContainerRef }
];
__decorate([
    Input(),
    __metadata("design:type", CdkPortal)
], TooltipDirective.prototype, "oavTooltip", void 0);
__decorate([
    HostListener('click', ['$event']),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [MouseEvent]),
    __metadata("design:returntype", void 0)
], TooltipDirective.prototype, "onClick", null);
__decorate([
    HostListener('mouseenter', ['$event']),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [MouseEvent]),
    __metadata("design:returntype", void 0)
], TooltipDirective.prototype, "onMouseEnter", null);
__decorate([
    HostListener('mouseleave', ['$event']),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [MouseEvent]),
    __metadata("design:returntype", void 0)
], TooltipDirective.prototype, "onMouseLeave", null);
__decorate([
    HostListener('window:click'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], TooltipDirective.prototype, "onWindowClick", null);
TooltipDirective = __decorate([
    Directive({
        selector: '[oavTooltip]'
    }),
    __metadata("design:paramtypes", [Overlay, ElementRef, ViewContainerRef])
], TooltipDirective);

const ajv = new Ajv({
    unknownFormats: 'ignore'
});
ajv.addFormat('int32', {
    type: 'number',
    validate: data => !isNaN(data) && data % 1 === 0 && data <= Math.pow(2, 31) && data >= Math.pow(2, 31) * -1
});
ajv.addFormat('int64', {
    type: 'number',
    validate: data => !isNaN(data) && data % 1 === 0
});
ajv.addFormat('float', {
    type: 'number',
    validate: data => !isNaN(data)
});
ajv.addFormat('double', {
    type: 'number',
    validate: data => !isNaN(data)
});
ajv.addFormat('byte', {
    type: 'string',
    validate: data => /^[0-9A-Za-z+/]*=?=?$/.test(data)
});
function getValidationFunction(schema) {
    return ajv.compile(schema);
}

var CodeInputComponent_1;
let CodeInputComponent = CodeInputComponent_1 = class CodeInputComponent {
    constructor(cd) {
        this.cd = cd;
        this.config = {};
        this.disabled = false;
        this.value = '';
        this.mode = 'text';
        this.minLines = 1;
        this.maxLines = 20;
        this.schema = null;
        this.errors = [];
        this.writing = false;
        this.validate = null;
        this.changes = new Subject();
        this.onChange = (v) => { };
        this.onTouched = () => { };
    }
    ngOnInit() {
        this.changes.pipe(debounceTime(100)).subscribe(() => this.checkValue());
    }
    ngOnDestroy() {
        this.changes.complete();
    }
    ngOnChanges(changes) {
        if (this.schema) {
            try {
                this.validate = getValidationFunction(this.schema);
            }
            catch (e) {
                console.error('Failed to get validation function for schema', this.schema, e);
                this.validate = null;
                this.errors = [e.message];
            }
        }
        else {
            this.validate = null;
            this.errors = [];
        }
        this.updateConfig();
    }
    updateConfig() {
        this.config = {
            minLines: this.minLines,
            maxLines: this.maxLines,
            wrap: true,
            tabSize: 2
        };
        if (this.readonly || this.readonly === '') {
            this.config.readOnly = true;
        }
    }
    registerOnChange(fn) {
        this.onChange = fn;
    }
    registerOnTouched(fn) {
        this.onTouched = fn;
    }
    setDisabledState(isDisabled) {
        this.disabled = isDisabled;
    }
    writeValue(obj) {
        this.value = obj;
        this.writing = true;
        try {
            this.cd.detectChanges();
        }
        catch (e) {
            console.error('Update Ace error', this.value, e);
        }
        this.writing = false;
        this.changes.next();
    }
    updateValue(value) {
        if (!this.writing) {
            this.value = value;
            this.changes.next();
            this.onChange(value);
        }
    }
    checkValue() {
        if (this.validate && this.mode === 'json') {
            this.errors = [];
            if (!this.value) {
                return;
            }
            try {
                const text = JSON.parse(this.value);
                if (!this.validate(text)) {
                    this.errors = this.validate.errors.map(e => `${e.dataPath}: ${e.keyword} ${e.message}`);
                }
            }
            catch (e) {
                this.errors = [e.message];
            }
        }
    }
};
CodeInputComponent.ctorParameters = () => [
    { type: ChangeDetectorRef }
];
__decorate([
    Input(),
    __metadata("design:type", Object)
], CodeInputComponent.prototype, "readonly", void 0);
__decorate([
    Input(),
    __metadata("design:type", String)
], CodeInputComponent.prototype, "mode", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], CodeInputComponent.prototype, "minLines", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], CodeInputComponent.prototype, "maxLines", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], CodeInputComponent.prototype, "schema", void 0);
CodeInputComponent = CodeInputComponent_1 = __decorate([
    Component({
        selector: 'oav-code-input',
        template: "<div>\n  <ace [config]=\"config\" [mode]=\"mode\" [value]=\"value\" (valueChange)=\"updateValue($event)\"></ace>\n</div>\n<div *ngFor=\"let err of errors\" class=\"oav-error\">{{ err }}</div>\n",
        changeDetection: ChangeDetectionStrategy.OnPush,
        providers: [
            {
                provide: NG_VALUE_ACCESSOR,
                useExisting: forwardRef(() => CodeInputComponent_1),
                multi: true
            }
        ]
    }),
    __metadata("design:paramtypes", [ChangeDetectorRef])
], CodeInputComponent);

function getLabel(op, pathItem, fields) {
    for (const field of fields) {
        if (field === 'path') {
            return pathItem.path;
        }
        if (op.operation[field]) {
            return op.operation[field];
        }
    }
    return '';
}

let IndexNavEndpointsComponent = class IndexNavEndpointsComponent {
    constructor(oavSettings) {
        this.oavSettings = oavSettings;
        if (!this.oavSettings) {
            this.oavSettings = defaultOavSettings;
        }
        this.showHoverLabel = !!this.oavSettings.indexHoverLabel;
    }
    getPrimaryLabel(op, pathItem) {
        return getLabel(op, pathItem, this.oavSettings.indexPrimaryLabel);
    }
    getHoverLabel(op, pathItem) {
        return getLabel(op, pathItem, this.oavSettings.indexHoverLabel);
    }
};
IndexNavEndpointsComponent.ctorParameters = () => [
    { type: OavSettings, decorators: [{ type: Optional }] }
];
__decorate([
    Input(),
    __metadata("design:type", Object)
], IndexNavEndpointsComponent.prototype, "tag", void 0);
IndexNavEndpointsComponent = __decorate([
    Component({
        selector: 'oav-index-nav-endpoints',
        template: "<ng-container *ngFor=\"let pathItem of tag.paths\">\n  <ng-container *ngFor=\"let op of pathItem.operations\">\n    <li>\n      <a [routerLink]=\"tag.tag.name + '/' + op.operation.operationId\" routerLinkActive=\"active\">\n        <span class=\"label\">\n          <oav-method class=\"font-size-smaller\" [method]=\"op.method\"></oav-method>\n          <span [class.deprecated]=\"op.operation.deprecated\">{{ getPrimaryLabel(op, pathItem) }}</span>\n        </span>\n        <span class=\"info-overlay\" *ngIf=\"showHoverLabel\">\n          <oav-method class=\"font-size-smaller\" [method]=\"op.method\"></oav-method>\n          <span [class.deprecated]=\"op.operation.deprecated\">{{ getHoverLabel(op, pathItem) }}</span>\n        </span>\n      </a>\n    </li>\n  </ng-container>\n</ng-container>\n",
        changeDetection: ChangeDetectionStrategy.OnPush
    }),
    __param(0, Optional()),
    __metadata("design:paramtypes", [OavSettings])
], IndexNavEndpointsComponent);

let OverviewViewEndpointsComponent = class OverviewViewEndpointsComponent {
    constructor(oavSettings) {
        this.oavSettings = oavSettings;
        if (!this.oavSettings) {
            this.oavSettings = defaultOavSettings;
        }
    }
    getLabel(op, pathItem) {
        return getLabel(op, pathItem, this.oavSettings.overviewPrimaryLabel);
    }
};
OverviewViewEndpointsComponent.ctorParameters = () => [
    { type: OavSettings, decorators: [{ type: Optional }] }
];
__decorate([
    Input(),
    __metadata("design:type", Object)
], OverviewViewEndpointsComponent.prototype, "tag", void 0);
OverviewViewEndpointsComponent = __decorate([
    Component({
        selector: 'oav-overview-view-endpoints',
        template: "<ng-container *ngFor=\"let pathItem of tag.paths\">\n  <ng-container *ngFor=\"let op of pathItem.operations\">\n    <h3>\n      <oav-method [method]=\"op.method\"></oav-method>\n      <oav-auth-status [operation]=\"op.operation\"></oav-auth-status>\n      <a [routerLink]=\"tag.tag.name + '/' + op.operation.operationId\" routerLinkActive=\"active\" class=\"oav-header-link ml-1\">\n        <span class=\"oav-path\" [class.deprecated]=\"op.operation.deprecated\">{{ pathItem.path }}</span>\n        <span class=\"oav-summary font-size-smaller ml-2\">{{ getLabel(op, pathItem) }}</span>\n      </a>\n    </h3>\n  </ng-container>\n</ng-container>\n",
        changeDetection: ChangeDetectionStrategy.OnPush
    }),
    __param(0, Optional()),
    __metadata("design:paramtypes", [OavSettings])
], OverviewViewEndpointsComponent);

function identifySchemaInfo(defaultName, schema) {
    const additionalModels = [];
    const modelInfo = identifyModelInfo(defaultName, readSchema(schema), additionalModels);
    return Object.assign(Object.assign({}, modelInfo), { additionalModels });
}
function identifyModelInfo(defaultName, schema, additionalModels) {
    const info = {
        name: defaultName,
        type: readType(schema),
        schema: readSchema(schema),
        properties: null
    };
    const modelName = readModelName(info.schema);
    if (modelName) {
        info.name = modelName;
    }
    if (info.type === 'object' || info.type === 'array') {
        info.properties = readProperties(info.schema, additionalModels);
    }
    return info;
}
function readProperties(schema, additionalModels) {
    const properties = [];
    if (schema.properties) {
        properties.push(...readPropertiesOfObject(schema));
    }
    if (schema.additionalProperties) {
        properties.push(readProperty('...', schema.additionalProperties));
    }
    if (Array.isArray(schema.items)) {
        for (let i = 0; i < schema.items.length; i++) {
            properties.push(readProperty(i.toString(), schema.items[i]));
        }
    }
    else if (schema.items) {
        properties.push(readProperty('', schema.items));
    }
    if (schema.additionalItems) {
        properties.push(readProperty('...', schema.additionalItems));
    }
    for (const prop of properties) {
        const identifiedModel = updateModelsFromProperty(prop.name + 'Object', prop.schema, additionalModels);
        prop.modelName = identifiedModel;
        if (!identifiedModel && (prop.type === 'object' || prop.type === 'array')) {
            prop.properties = readProperties(prop.schema, additionalModels);
        }
    }
    return properties;
}
function updateModelsFromProperty(defaultName, schema, additionalModels) {
    if (schema.$ref) {
        // Unknown schema, probably did not resolved because of recursive structure
        const parts = schema.$ref.split('/');
        return parts.pop();
    }
    const refName = readModelName(schema);
    const type = readType(schema);
    if (type === 'object') {
        if (!schema.properties && !schema.additionalProperties) {
            return null;
        }
        const modelName = refName || defaultName;
        if (!additionalModels.find(m => m.name === modelName)) {
            const designatedIndex = additionalModels.length;
            const modelInfo = identifyModelInfo(modelName, schema, additionalModels);
            additionalModels.splice(designatedIndex, 0, modelInfo);
        }
        return modelName;
    }
    return null;
}
function readSchema(schema) {
    if (typeof schema === 'boolean') {
        return { type: 'boolean' };
    }
    return schema;
}
function readType(schema) {
    if (schema && typeof schema === 'object' && schema.type) {
        return Array.isArray(schema.type) ? schema.type[0] : schema.type;
    }
    if (typeof schema === 'boolean') {
        return 'boolean';
    }
    if (schema.properties || schema.additionalProperties) {
        return 'object';
    }
    if (schema.items || schema.additionalItems) {
        return 'array';
    }
    return null;
}
function readModelName(schema) {
    if (schema.$$ref) {
        const refParts = schema.$$ref.split('/');
        return refParts.pop();
    }
    return undefined;
}
function readPropertiesOfObject(schema) {
    if (schema.properties) {
        const requiredFields = schema.required || [];
        return Object.keys(schema.properties).map(name => readProperty(name, schema.properties[name], requiredFields.includes(name)));
    }
    return [];
}
function readProperty(name, schema, required = false) {
    const s = readSchema(schema);
    return { name, type: readType(s), schema: s, required };
}

let JsonSchemaComponent = class JsonSchemaComponent {
    constructor() { }
    ngOnChanges(changes) {
        const info = identifySchemaInfo(this.defaultModelName, this.schema);
        info.open = true;
        this.models = [info, ...info.additionalModels];
    }
    isPrimitive(type) {
        return type === 'integer' || type === 'number' || type === 'string' || type === 'boolean' || type === 'null' || type === 'any';
    }
    toggleModel(model) {
        if (model.open) {
            model.open = false;
        }
        else {
            model.open = true;
        }
    }
    toggleModelByName(name) {
        const modelInfo = this.models.find(m => m.name === name);
        if (modelInfo) {
            this.toggleModel(modelInfo);
        }
    }
};
__decorate([
    Input(),
    __metadata("design:type", Object)
], JsonSchemaComponent.prototype, "schema", void 0);
__decorate([
    Input(),
    __metadata("design:type", String)
], JsonSchemaComponent.prototype, "defaultModelName", void 0);
JsonSchemaComponent = __decorate([
    Component({
        selector: 'oav-json-schema',
        template: "<div class=\"oav-json-schema-description ml-0\" *ngIf=\"schema?.description\">\n  <markdown [data]=\"schema?.description\"></markdown>\n</div>\n<div *ngFor=\"let model of models\">\n  <ng-container *ngIf=\"model.type === 'object'\">\n    <div *ngIf=\"!model.open\" class=\"oav-json-schema-code\">\n      <a href=\"javascript:\" (click)=\"toggleModel(model)\" class=\"oav-json-schema-toggle\">\n        {{ model.name }}\n      </a>\n      {{ '{' }} <a href=\"javascript:\" (click)=\"toggleModel(model)\" class=\"oav-json-schema-toggle\">...</a> {{ '}' }}\n    </div>\n    <ng-container *ngIf=\"model.open\">\n      <div class=\"oav-json-schema-code\">\n        <a href=\"javascript:\" (click)=\"toggleModel(model)\" class=\"oav-json-schema-toggle\">\n          {{ model.name }}\n        </a>\n        {{ '{' }}\n      </div>\n      <div class=\"ml-3 json-schema-property\" *ngFor=\"let prop of model.properties\">\n        <div class=\"oav-json-schema-name oav-json-schema-code\" [class.oav-json-schema-required]=\"prop.required\">{{ prop.name }}:</div>\n        <div class=\"oav-json-schema-type\" *ngIf=\"!prop.modelName\">\n          <ng-container *ngTemplateOutlet=\"typeOutput; context: prop\"></ng-container>\n        </div>\n        <div class=\"oav-json-schema-type oav-json-schema-code\" *ngIf=\"prop.modelName\">\n          <a href=\"javascript:\" (click)=\"toggleModelByName(prop.modelName)\" class=\"oav-json-schema-toggle\">\n            {{ prop.modelName }}\n          </a>\n        </div>\n        <div class=\"oav-json-schema-description\">\n          <markdown *ngIf=\"prop.schema?.description\" [data]=\"prop.schema?.description\"></markdown>\n        </div>\n      </div>\n      <div class=\"oav-json-schema-code\">{{ '}' }}</div>\n    </ng-container>\n  </ng-container>\n  <ng-container *ngIf=\"model.type === 'array'\">\n    <ng-container *ngTemplateOutlet=\"arrayOutput; context: model\"></ng-container>\n  </ng-container>\n  <ng-container *ngIf=\"isPrimitive(model.type)\">\n    <ng-container *ngTemplateOutlet=\"typeOutput; context: model\"></ng-container>\n  </ng-container>\n</div>\n\n<ng-template #arrayOutput let-properties=\"properties\">\n  <ng-container *ngIf=\"properties?.length === 1 && properties[0] as prop; else multipleProps\">\n    <div class=\"oav-json-schema-code\" *ngIf=\"prop.modelName\">\n      [\n      <a href=\"javascript:\" (click)=\"toggleModelByName(prop.modelName)\" class=\"oav-json-schema-toggle\">\n        {{ prop.modelName }}\n      </a>\n      ]\n    </div>\n    <div class=\"oav-json-schema-code\" *ngIf=\"!prop.modelName\">\n      [ <span class=\"oav-json-schema-type-name\">{{ prop.type }}</span> ]\n    </div>\n  </ng-container>\n  <ng-template #multipleProps>\n    <div class=\"oav-json-schema-code\">[</div>\n    <div class=\"ml-3 json-schema-property\" *ngFor=\"let prop of properties\">\n      <div class=\"oav-json-schema-name oav-json-schema-code\" *ngIf=\"prop.name\">{{ prop.name }}:</div>\n      <div class=\"oav-json-schema-type\" *ngIf=\"!prop.modelName\">\n        <ng-container *ngTemplateOutlet=\"typeOutput; context: prop\"></ng-container>\n      </div>\n      <div class=\"oav-json-schema-type oav-json-schema-code\" *ngIf=\"prop.modelName\">\n        <a href=\"javascript:\" (click)=\"toggleModelByName(prop.modelName)\" class=\"oav-json-schema-toggle\">\n          {{ prop.modelName }}\n        </a>\n      </div>\n      <div class=\"oav-json-schema-description\">\n        <markdown *ngIf=\"prop.schema?.description\" [data]=\"prop.schema?.description\"></markdown>\n      </div>\n    </div>\n    <div class=\"oav-json-schema-code\">]</div>\n  </ng-template>\n</ng-template>\n\n<ng-template #typeOutput let-type=\"type\" let-schema=\"schema\" let-properties=\"properties\">\n  <ng-container *ngIf=\"type === 'array'\">\n    <ng-container *ngTemplateOutlet=\"arrayOutput; context: { properties: properties }\"></ng-container>\n  </ng-container>\n  <ng-container *ngIf=\"type !== 'array'\">\n    <span class=\"oav-json-schema-type-name oav-json-schema-code\">\n      {{ type }}\n    </span>\n    <span class=\"oav-json-schema-format oav-json-schema-code\" *ngIf=\"schema?.format\">({{ schema?.format }})</span>\n    <span *ngIf=\"schema?.enum?.length\" class=\"oav-json-schema-code\">({{ schema.enum.join(' | ') }})</span>\n  </ng-container>\n</ng-template>\n",
        changeDetection: ChangeDetectionStrategy.OnPush
    }),
    __metadata("design:paramtypes", [])
], JsonSchemaComponent);

let OpenapiViewerModule = class OpenapiViewerModule {
};
OpenapiViewerModule = __decorate([
    NgModule({
        declarations: [
            OpenapiViewerComponent,
            OperationComponent,
            ParameterComponent,
            ResponsesComponent,
            MultiItemsInputComponent,
            RequestsListComponent,
            RequestViewComponent,
            OpenableDirective,
            MethodHeaderComponent,
            MethodComponent,
            IndexNavComponent,
            OperationViewComponent,
            OverviewViewComponent,
            TabNavComponent,
            TabNavItemDirective,
            TemplateOutletComponent,
            RequestBodyInputComponent,
            AuthenticationViewComponent,
            AuthCredentialsComponent,
            AuthStatusComponent,
            TooltipComponent,
            TooltipDirective,
            CodeInputComponent,
            IndexNavEndpointsComponent,
            OverviewViewEndpointsComponent,
            ModelsViewComponent,
            JsonSchemaComponent
        ],
        imports: [
            CommonModule,
            OpenapiViewerRoutingModule,
            FormsModule,
            ReactiveFormsModule,
            HttpClientModule,
            MarkdownModule.forRoot(),
            PortalModule,
            OverlayModule,
            AceModule
        ],
        exports: [OpenapiViewerComponent, OperationComponent]
    })
], OpenapiViewerModule);

/*
 * Public API Surface of openapi-viewer
 */

/**
 * Generated bundle index. Do not edit.
 */

export { OavSettings, OpenapiAuthService, OpenapiViewerComponent, OpenapiViewerModule, OpenapiViewerService, OperationComponent, buildAuthorizationUrl, defaultOavSettings, getAuthorizationBaseUrl, getTagsOfPath, getTokenBaseUrl, identifyFlows, ParameterComponent as ɵa, ResponsesComponent as ɵb, MultiItemsInputComponent as ɵc, RequestsListComponent as ɵd, RequestViewComponent as ɵe, OpenableDirective as ɵf, MethodHeaderComponent as ɵg, MethodComponent as ɵh, IndexNavComponent as ɵi, OperationViewComponent as ɵj, OverviewViewComponent as ɵk, TabNavComponent as ɵl, TabNavItemDirective as ɵm, TemplateOutletComponent as ɵn, RequestBodyInputComponent as ɵo, AuthenticationViewComponent as ɵp, AuthCredentialsComponent as ɵq, AuthStatusComponent as ɵr, TooltipComponent as ɵs, TooltipDirective as ɵt, CodeInputComponent as ɵu, IndexNavEndpointsComponent as ɵv, OverviewViewEndpointsComponent as ɵw, ModelsViewComponent as ɵx, JsonSchemaComponent as ɵy, OpenapiViewerRoutingModule as ɵz };
//# sourceMappingURL=dhcode-openapi-viewer.js.map
